########################################################################
#
#       License: BSD
#       Created: August 15, 2012
#       Author:  Francesc Alted - francesc@blosc.io
#
########################################################################

from __future__ import absolute_import

import tempfile
import os
import os.path
import sys
import glob
import shutil

if sys.version < "2.7":
    import unittest2 as unittest
    from unittest2 import TestCase, skipUnless, SkipTest
else:
    import unittest  # noqa
    from unittest import TestCase, skipUnless, SkipTest  # noqa


# Global variables for the tests
verbose = False
heavy = False


def remove_tree(rootdir):
    # Remove every directory starting with rootdir
    for dir_ in glob.glob(rootdir+'*'):
        shutil.rmtree(dir_)


# Useful superclass for disk-based tests
class MayBeDiskTest():

    disk = False

    def setUp(self):
        if self.disk:
            prefix = 'bcolz-' + self.__class__.__name__
            self.rootdir = tempfile.mkdtemp(prefix=prefix)
            os.rmdir(self.rootdir)  # tests needs this cleared
        else:
            self.rootdir = None

    def tearDown(self):
        if self.disk:
            # Remove every directory starting with rootdir
            for dir_ in glob.glob(self.rootdir+'*'):
                shutil.rmtree(dir_)



# #######################################################################
#
#       License: BSD
#       Created: December 14, 2010
#       Author:  Francesc Alted - francesc@blosc.io
#
########################################################################

import sys
import os
import glob
import textwrap
from distutils.core import Extension
from distutils.dep_util import newer

from paver.easy import *
from paver.setuputils import setup


# Some functions for showing errors and warnings.
def _print_admonition(kind, head, body):
    tw = textwrap.TextWrapper(
        initial_indent='   ', subsequent_indent='   ')

    print(".. %s:: %s" % (kind.upper(), head))
    for line in tw.wrap(body):
        print line


def exit_with_error(head, body=''):
    _print_admonition('error', head, body)
    sys.exit(1)


def print_warning(head, body=''):
    _print_admonition('warning', head, body)


def check_import(pkgname, pkgver):
    try:
        mod = __import__(pkgname)
    except ImportError:
        exit_with_error(
            "You need %(pkgname)s %(pkgver)s or greater to run bcolz!"
            % {'pkgname': pkgname, 'pkgver': pkgver})
    else:
        if mod.__version__ < pkgver:
            exit_with_error(
                "You need %(pkgname)s %(pkgver)s or greater to run bcolz!"
                % {'pkgname': pkgname, 'pkgver': pkgver})

    print ( "* Found %(pkgname)s %(pkgver)s package installed."
            % {'pkgname': pkgname, 'pkgver': mod.__version__} )
    globals()[pkgname] = mod


########### Check versions ##########

# Check for Python
if not (sys.version_info[0] >= 2 and sys.version_info[1] >= 6):
    exit_with_error("You need Python 2.6 or greater to install bcolz!")

# The minimum version of Cython required for generating extensions
min_cython_version = '0.20'
# The minimum version of NumPy required
min_numpy_version = '1.5'
# The minimum version of Numexpr (optional)
min_numexpr_version = '1.4.1'

# Check for Cython
cython = False
try:
    from Cython.Compiler.Main import Version

    if Version.version >= min_cython_version:
        cython = True
except:
    pass


# Check for NumPy
check_import('numpy', min_numpy_version)
# Check for Numexpr
check_import('numexpr', min_numexpr_version)


########### End of version checks ##########

# bcolz version
VERSION = open('VERSION').read().strip()
# Create the version.py file
open('bcolz/version.py', 'w').write('__version__ = "%s"\n' % VERSION)


# Global variables
CFLAGS = os.environ.get('CFLAGS', '').split()
LFLAGS = os.environ.get('LFLAGS', '').split()
lib_dirs = []
libs = []
inc_dirs = ['bcolz', 'blosc']
# Include NumPy header dirs
from numpy.distutils.misc_util import get_numpy_include_dirs

inc_dirs.extend(get_numpy_include_dirs())
cython_pyxfiles = glob.glob('bcolz/*.pyx')
cython_cfiles = [fn.split('.')[0] + '.c' for fn in cython_pyxfiles]
blosc_files = glob.glob('blosc/*.c')

# Handle --lflags=[FLAGS] --cflags=[FLAGS]
args = sys.argv[:]
for arg in args:
    if arg.find('--lflags=') == 0:
        LFLAGS = arg.split('=')[1].split()
        sys.argv.remove(arg)
    elif arg.find('--cflags=') == 0:
        CFLAGS = arg.split('=')[1].split()
        sys.argv.remove(arg)

# Add -msse2 flag for optimizing shuffle in Blosc
if os.name == 'posix':
    CFLAGS.append("-msse2")


# Paver tasks
@task
def cythonize():
    for fn in glob.glob('bcolz/*.pyx'):
        dest = fn.split('.')[0] + '.c'
        if newer(fn, dest):
            if not cython:
                exit_with_error(
                    "Need Cython >= %s to generate extensions."
                    % min_cython_version)
            sh("cython " + fn)


@task
@needs('html', 'setuptools.command.sdist')
def sdist():
    """Generate a source distribution for the package."""
    pass


@task
@needs(['cythonize', 'setuptools.command.build'])
def build():
    pass


@task
@needs(['cythonize', 'setuptools.command.build_ext'])
def build_ext():
    pass


@task
@needs('paver.doctools.html')
def html(options):
    """Build the docs in HTML format."""
    destdir = path("doc/html")
    destdir.rmtree()
    builtdocs = path("doc") / options.builddir / "html"
    builtdocs.move(destdir)


@task
def pdf(options):
    """Build the docs in PDF format."""
    dest = path("doc") / "bcolz-manual.pdf"
    sh("cd doc; make latexpdf")
    builtdocs = path("doc") / options.builddir / "latex" / "bcolz.pdf"
    builtdocs.move(dest)


# Options for Paver tasks
options(

    sphinx=Bunch(
        docroot="doc",
        builddir="_build"
    ),

)

classifiers = """\
Development Status :: 4 - Beta
Intended Audience :: Developers
Intended Audience :: Information Technology
Intended Audience :: Science/Research
License :: OSI Approved :: BSD License
Programming Language :: Python
Topic :: Software Development :: Libraries :: Python Modules
Operating System :: Microsoft :: Windows
Operating System :: Unix
"""

# Package options
setup(
    name='bcolz',
    version=VERSION,
    description='A columnar and compressed data container.',
    long_description="""\

bcolz is a columnar and compressed data container.  Column storage allows
for efficiently querying tables with a large number of columns.  It also
allows for cheap addition and removal of column.  In addition, bcolz objects
are compressed by default for reducing memory/disk I/O needs.  The
compression process is carried out internally by Blosc, a high-performance
compressor that is optimized for binary data.

""",
    classifiers=filter(None, classifiers.split("\n")),
    author='Francesc Alted',
    author_email='francesc@blosc.io',
    url="https://github.com/Blosc/bcolz",
    license='http://www.opensource.org/licenses/bsd-license.php',
    # It is better to upload manually to PyPI
    #download_url = "http://bcolz.blosc.org/download/bcolz-%s/bcolz-%s.tar
    # .gz" % (VERSION, VERSION),
    platforms=['any'],
    ext_modules=[
        Extension("bcolz.carray",
                  include_dirs=inc_dirs,
                  sources=cython_cfiles + blosc_files,
                  depends=["bcolz/definitions.pxd"] + blosc_files,
                  library_dirs=lib_dirs,
                  libraries=libs,
                  extra_link_args=LFLAGS,
                  extra_compile_args=CFLAGS),
    ],
    packages=['bcolz', 'bcolz.tests'],
    include_package_data=True,

)




# Benchmark for assessing the `fromiter()` speed.

from time import time

import numpy as np
from numpy.testing import assert_array_equal

import bcolz
from bcolz.py2help import xrange, izip


N = int(1e6)  # the number of elements in x
clevel = 2  # the compression level

print("Creating inputs with %d elements..." % N)

x = xrange(N)  # not a true iterable, but can be converted
y = xrange(1, N + 1)
z = xrange(2, N + 2)

print("Starting benchmark now for creating arrays...")
# Create a ndarray
# x = (i for i in xrange(N))    # true iterable
t0 = time()
out = np.fromiter(x, dtype='f8', count=N)
print("Time for array--> %.3f" % (time() - t0,))
print("out-->", len(out))

#bcolz.set_num_threads(bcolz.ncores//2)

# Create a carray
#x = (i for i in xrange(N))    # true iterable
t0 = time()
cout = bcolz.fromiter(x, dtype='f8', count=N, cparams=bcolz.cparams(clevel))
print("Time for carray--> %.3f" % (time() - t0,))
print("cout-->", len(cout))
assert_array_equal(out, cout, "Arrays are not equal")

# Create a carray (with unknown size)
#x = (i for i in xrange(N))    # true iterable
t0 = time()
cout = bcolz.fromiter(x, dtype='f8', count=-1, cparams=bcolz.cparams(clevel))
print("Time for carray (count=-1)--> %.3f" % (time() - t0,))
print("cout-->", len(cout))
assert_array_equal(out, cout, "Arrays are not equal")

# Retrieve from a structured ndarray
gen = ((i, j, k) for i, j, k in izip(x, y, z))
t0 = time()
out = np.fromiter(gen, dtype="f8,f8,f8", count=N)
print("Time for structured array--> %.3f" % (time() - t0,))
print("out-->", len(out))

# Retrieve from a ctable
gen = ((i, j, k) for i, j, k in izip(x, y, z))
t0 = time()
cout = bcolz.fromiter(gen, dtype="f8,f8,f8", count=N)
print("Time for ctable--> %.3f" % (time() - t0,))
print("out-->", len(cout))
assert_array_equal(out, cout[:], "Arrays are not equal")

# Retrieve from a ctable (with unknown size)
gen = ((i, j, k) for i, j, k in izip(x, y, z))
t0 = time()
cout = bcolz.fromiter(gen, dtype="f8,f8,f8", count=-1)
print("Time for ctable (count=-1)--> %.3f" % (time() - t0,))
print("out-->", len(cout))
assert_array_equal(out, cout[:], "Arrays are not equal")



########################################################################
#
#       License: BSD
#       Created: January 18, 2011
#       Author:  Francesc Alted - francesc@blosc.io
#
########################################################################

from __future__ import absolute_import


import numpy as np
import bcolz
from bcolz.py2help import xrange
from bcolz.tests.common import (
    MayBeDiskTest, TestCase, unittest)


class listTest(MayBeDiskTest):

    def test00a(self):
        """Testing wheretrue() in combination with a list constructor"""
        a = bcolz.zeros(self.N, dtype="bool", rootdir=self.rootdir)
        a[30:40] = bcolz.ones(10, dtype="bool")
        alist = list(a)
        blist1 = [r for r in a.wheretrue()]
        self.assertTrue(blist1 == list(range(30, 40)))
        alist2 = list(a)
        self.assertTrue(alist == alist2, "wheretrue() not working correctly")

    def test00b(self):
        """Testing wheretrue() with a multidimensional array"""
        a = bcolz.zeros((self.N, 10), dtype="bool", rootdir=self.rootdir)
        a[30:40] = bcolz.ones(10, dtype="bool")
        self.assertRaises(NotImplementedError, a.wheretrue)

    def test01a(self):
        """Testing where() in combination with a list constructor"""
        a = bcolz.zeros(self.N, dtype="bool", rootdir=self.rootdir)
        a[30:40] = bcolz.ones(10, dtype="bool")
        b = bcolz.arange(self.N, dtype="f4")
        blist = list(b)
        blist1 = [r for r in b.where(a)]
        self.assertTrue(blist1 == list(range(30, 40)))
        blist2 = list(b)
        self.assertTrue(blist == blist2, "where() not working correctly")

    def test01b(self):
        """Testing where() with a multidimensional array"""
        a = bcolz.zeros((self.N, 10), dtype="bool", rootdir=self.rootdir)
        a[30:40] = bcolz.ones(10, dtype="bool")
        b = bcolz.arange(self.N * 10, dtype="f4").reshape((self.N, 10))
        self.assertRaises(NotImplementedError, b.where, a)

    def test02(self):
        """Testing iter() in combination with a list constructor"""
        b = bcolz.arange(self.N, dtype="f4", rootdir=self.rootdir)
        blist = list(b)
        blist1 = [r for r in b.iter(3, 10)]
        self.assertTrue(blist1 == list(range(3, 10)))
        blist2 = list(b)
        self.assertTrue(blist == blist2, "iter() not working correctly")


class small_listTest(listTest, TestCase):
    N = 100


class big_listTest(listTest, TestCase):
    N = 10000


class small_listDiskTest(listTest, TestCase):
    N = 100
    disk = True


class big_listDiskTest(listTest, TestCase):
    N = 10000
    disk = True


class whereblocksTest(MayBeDiskTest):

    def test00(self):
        """Testing `whereblocks` method with only an expression"""
        N = self.N
        ra = np.fromiter(((i, i * 2., i * 3)
                          for i in xrange(N)), dtype='i4,f8,i8')
        t = bcolz.ctable(ra)
        l, s = 0, 0
        for block in t.whereblocks('f1 < f2'):
            l += len(block)
            s += block['f0'].sum()
        self.assertEqual(l, N - 1)
        self.assertEqual(s, (N - 1) * (N / 2))  # Gauss summation formula

    def test01(self):
        """Testing `whereblocks` method with a `blen`"""
        N = self.N
        ra = np.fromiter(((i, i * 2., i * 3)
                          for i in xrange(N)), dtype='i4,f8,i8')
        t = bcolz.ctable(ra)
        l, s = 0, 0
        for block in t.whereblocks('f0 <= f1', blen=100):
            l += len(block)
            # All blocks should be of length 100, except the last one,
            # which should be 0 or 20
            self.assertTrue(len(block) in (0, 20, 100))
            s += block['f0'].sum()
        self.assertEqual(l, N)
        self.assertEqual(s, (N - 1) * (N / 2))  # Gauss summation formula

    def test02(self):
        """Testing `whereblocks` method with a `outfields` with 2 fields"""
        N = self.N
        ra = np.fromiter(((i, i, i * 3) for i in xrange(N)), dtype='i4,f8,i8')
        t = bcolz.ctable(ra)
        l, s = 0, 0
        for block in t.whereblocks('f1 < f2', outfields=('f1', 'f2')):
            self.assertEqual(block.dtype.names, ('f1', 'f2'))
            l += len(block)
            s += block['f1'].sum()
        self.assertEqual(l, N - 1)
        self.assertEqual(s, (N - 1) * (N / 2))  # Gauss summation formula

    def test03(self):
        """Testing `whereblocks` method with a `outfields` with 1 field"""
        N = self.N
        ra = np.fromiter(((i, i, i * 3) for i in xrange(N)), dtype='i4,f8,i8')
        t = bcolz.ctable(ra)
        l, s = 0, 0
        for block in t.whereblocks('f1 < f2', outfields=('f1',)):
            self.assertEqual(block.dtype.names, ('f1',))
            l += len(block)
            s += block['f1'].sum()
        self.assertEqual(l, N - 1)
        self.assertEqual(s, (N - 1) * (N / 2))  # Gauss summation formula

    def test04(self):
        """Testing `whereblocks` method with a `limit` parameter"""
        N, M = self.N, 101
        ra = np.fromiter(((i, i * 2., i * 3)
                          for i in xrange(N)), dtype='i4,f8,i8')
        t = bcolz.ctable(ra)
        l, s = 0, 0
        for block in t.whereblocks('f1 < f2', limit=M):
            l += len(block)
            s += block['f0'].sum()
        self.assertEqual(l, M)
        self.assertEqual(s, M * ((M + 1) / 2))  # Gauss summation formula

    def test05(self):
        """Testing `whereblocks` method with a `limit` parameter"""
        N, M = self.N, 101
        ra = np.fromiter(((i, i * 2., i * 3)
                          for i in xrange(N)), dtype='i4,f8,i8')
        t = bcolz.ctable(ra)
        l, s = 0, 0
        for block in t.whereblocks('f1 < f2', limit=M):
            l += len(block)
            s += block['f0'].sum()
        self.assertEqual(l, M)
        self.assertEqual(s, M * ((M + 1) / 2))  # Gauss summation formula

    def test06(self):
        """Testing `whereblocks` method with a `skip` parameter"""
        N, M = self.N, 101
        ra = np.fromiter(((i, i * 2., i * 3)
                          for i in xrange(N)), dtype='i4,f8,i8')
        t = bcolz.ctable(ra)
        l, s = 0, 0
        for block in t.whereblocks('f1 < f2', skip=N - M):
            l += len(block)
            s += block['f0'].sum()
        self.assertEqual(l, M - 1)
        self.assertEqual(s, np.arange(N - M + 1, N).sum())

    def test07(self):
        """Testing `whereblocks` method with a `limit`, `skip` parameter"""
        N, M = self.N, 101
        ra = np.fromiter(((i, i * 2., i * 3)
                          for i in xrange(N)), dtype='i4,f8,i8')
        t = bcolz.ctable(ra)
        l, s = 0, 0
        for block in t.whereblocks('f1 < f2', limit=N - M - 2, skip=M):
            l += len(block)
            s += block['f0'].sum()
        self.assertEqual(l, N - M - 2)
        self.assertEqual(s, np.arange(M + 1, N - 1).sum())


class small_whereblocksTest(whereblocksTest, TestCase):
    N = 120


class big_whereblocksTest(whereblocksTest, TestCase):
    N = 10000


class small_whereblocksDiskTest(whereblocksTest, TestCase):
    N = 120
    disk = True


class big_whereblocksDiskTest(whereblocksTest, TestCase):
    N = 10000
    disk = True


if __name__ == '__main__':
    unittest.main(verbosity=2)


# Local Variables:
# mode: python
# py-indent-offset: 4
# tab-width: 4
# fill-column: 72
# End:



# -*- coding: utf-8 -*-
########################################################################
#
#       License: BSD
#       Created: January 2, 2014
#       Author:  Francesc Alted - francesc@blosc.org
#
########################################################################

from __future__ import absolute_import

"""
Unit tests related to the handling of arrays of objects
-------------------------------------------------------

Notes on object handling:

1. Only one dimensional arrays of objects are handled

2. Composite dtypes that contains objects are currently not handled.

"""

import unittest
from unittest import TestCase

import numpy as np
import bcolz
from bcolz.tests.common import MayBeDiskTest


class ObjectCarrayTest(MayBeDiskTest):
    def test_carray_1d_source(self):
        """Testing carray of objects, 1d source"""
        src_data = ['s'*i for i in range(10)]
        carr = bcolz.carray(src_data, dtype=np.dtype('O'))

        self.assertEqual(len(carr.shape), 1)
        self.assertEqual(len(src_data), carr.shape[0])
        for i in range(len(carr)):
            self.assertEqual(carr[i], src_data[i])
            self.assertEqual(carr[i], src_data[i])

    def test_carray_2d_source(self):
        """Testing carray of objects, 2d source

        Expected result will be a 1d carray whose elements are
        containers holding the inner dimension
        """
        src_data = [(i, 's'*i) for i in range(10)]
        carr = bcolz.carray(src_data, dtype=np.dtype('O'))
        # note that carray should always create a 1 dimensional
        # array of objects.
        self.assertEqual(len(carr.shape), 1)
        self.assertEqual(len(src_data), carr.shape[0])
        for i in range(len(carr)):
            self.assertEqual(carr[i][0], src_data[i][0])
            self.assertEqual(carr[i][1], src_data[i][1])

    def test_carray_tuple_source(self):
        """Testing a carray of objects that are tuples

        This uses a numpy container as source. Tuples should be
        preserved
        """
        src_data = np.empty((10,), dtype=np.dtype('O'))
        src_data[:] = [(i, 's'*i) for i in range(src_data.shape[0])]
        carr = bcolz.carray(src_data)
        self.assertEqual(len(carr.shape), 1)
        self.assertEqual(len(src_data), carr.shape[0])
        self.assertEqual(type(carr[0]), tuple)
        self.assertEqual(type(carr[0]), type(src_data[0]))
        for i in range(len(carr)):
            self.assertEqual(carr[i][0], src_data[i][0])
            self.assertEqual(carr[i][1], src_data[i][1])

    def test_carray_record(self):
        """Testing carray handling of record dtypes containing
        objects.  They must raise a type error exception, as they are
        not supported
        """
        src_data = [(i, 's'*i) for i in range(10)]
        self.assertRaises(TypeError, bcolz.carray,
                          src_data, dtype=np.dtype('O,O'))

    def test_carray_record_as_object(self):
        src_data = np.empty((10,), dtype=np.dtype('u1,O'))
        src_data[:] = [(i, 's'*i) for i in range(10)]
        carr = bcolz.carray(src_data, dtype=np.dtype('O'))
        self.assertEqual(len(carr.shape), 1)
        self.assertEqual(len(src_data), carr.shape[0])
        for i in range(len(carr)):
            self.assertEqual(carr[i][0], src_data[i][0])
            self.assertEqual(carr[i][1], src_data[i][1])

    # The following tests document different alternatives in handling
    # input data which would infer a record dtype in the resulting
    # carray.
    #
    # option 1: fail with a type error as if the dtype was
    # explicit
    #
    # option 2: handle it as an array of arrays of objects.
    def test_carray_record_inferred_opt1(self):
        """Testing carray handling of inferred record dtypes
        containing objects.  When there is no explicit dtype in the
        carray constructor, the dtype is inferred. This test checks
        that an inferred dtype results in a type error.
        """
        src_data = np.empty((10,), dtype=np.dtype('u1,O'))
        src_data[:] = [(i, 's'*i) for i in range(10)]
        self.assertRaises(TypeError, bcolz.carray, src_data)

    # This test is disabled.  option 1 above has been implemented.
    def _test_carray_record_inferred_opt2(self):
        """Testing carray handling of inferred record dtypes
        containing objects.  When there is no explicit dtype in the
        carray constructor, the dtype becomes 'O', and the carrays
        behaves accordingly (one dimensional)
        """
        src_data = np.empty((10,), dtype=np.dtype('u1,O'))
        src_data[:] = [(i, 's'*i) for i in range(10)]

        carr = bcolz.carray(src_data)
        # note: this is similar as if it was created with dtype='O'
        self.assertEqual(len(carr.shape), 1)
        self.assertEqual(len(src_data), carr.shape[0])
        for i in range(len(carr)):
            self.assertEqual(carr[i][0], src_data[i][0])
            self.assertEqual(carr[i][1], src_data[i][1])

    def test_create_unsafe_carray_with_unsafe_data(self):
        """ We introduce a safe keyword arg which removes dtype checking.
        We don't want this to interfere with creation.
        """
        b = bcolz.carray([1, 2, 3], dtype='i4', safe=False)
        self.assertEqual(b.safe, False)
        self.assertEqual(b[0], 1)


class ObjectCarraymemoryTest(ObjectCarrayTest, TestCase):
    disk = False


class ObjectCarrayDiskTest(ObjectCarrayTest, TestCase):
    disk = True


if __name__ == '__main__':
    unittest.main(verbosity=2)


# Local Variables:
# mode: python
# coding: utf-8
# fill-column: 78
# End:



########################################################################
#
#       License: BSD
#       Created: August 05, 2010
#       Author:  Francesc Alted - francesc@blosc.org
#
########################################################################

"""
bcolz: columnar and compressed data containers
==============================================

bcolz provides columnar and compressed data containers.  Column storage
allows for efficiently querying tables with a large number of columns.  It
also allows for cheap addition and removal of column.  In addition,
bcolz objects are compressed by default for reducing memory/disk I/O needs.
The compression process is carried out internally by Blosc,
a high-performance compressor that is optimized for binary data.

"""

min_numexpr_version = '1.4.1'  # the minimum version of Numexpr needed
numexpr_here = False
try:
    import numexpr
except ImportError:
    pass
else:
    if numexpr.__version__ >= min_numexpr_version:
        numexpr_here = True

# Check for pandas (for data container conversion purposes)
pandas_here = False
try:
    import pandas
except ImportError:
    pass
else:
    pandas_here = True

# Check for PyTables (for data container conversion purposes)
tables_here = False
try:
    import tables
except ImportError:
    pass
else:
    tables_here = True


# Print array functions (imported from NumPy)
from bcolz.arrayprint import (
    array2string, set_printoptions, get_printoptions )

from bcolz.carray_ext import (
    carray, blosc_version, blosc_compressor_list,
    _blosc_set_nthreads as blosc_set_nthreads,
    _blosc_init, _blosc_destroy)
from bcolz.ctable import ctable
from bcolz.toplevel import (
    print_versions, detect_number_of_cores, set_nthreads,
    open, fromiter, arange, zeros, ones, fill,
    iterblocks, cparams, walk)
from bcolz.chunked_eval import eval
from bcolz.defaults import defaults
from bcolz.version import __version__
try:
    from bcolz.tests import test
except ImportError:
    def test(*args, **kwargs):
        print("Could not import tests.\n"
              "If on Python2.6 please install unittest2")


def _get_git_descrtiption(path_):
    """ Get the output of git-describe when executed in a given path. """

    # imports in function because:
    # a) easier to refactor
    # b) clear they are only used here
    import subprocess
    import os
    import os.path as path
    from bcolz.py2help import check_output

    # make an absolute path if required, for example when running in a clone
    if not path.isabs(path_):
        path_ = path.join(os.getcwd(), path_)
    # look up the commit using subprocess and git describe
    try:
        # redirect stderr to stdout to make sure the git error message in case
        # we are not in a git repo doesn't appear on the screen and confuse the
        # user.
        label = check_output(["git", "describe"], cwd=path_,
                             stderr=subprocess.STDOUT).strip()
        return label
    except OSError:  # in case git wasn't found
        pass
    except subprocess.CalledProcessError:  # not in git repo
        pass

git_description = _get_git_descrtiption(__path__[0])

# Initialization code for the Blosc and numexpr libraries
_blosc_init()
ncores = detect_number_of_cores()
blosc_set_nthreads(ncores)
# Benchmarks show that using several threads can be an advantage in bcolz
blosc_set_nthreads(ncores)
if numexpr_here:
    numexpr.set_num_threads(ncores)
import atexit
atexit.register(_blosc_destroy)



# Benchmark for pickling carray/ctable objects

from __future__ import print_function

import os.path
import shutil
import contextlib
import time
import pickle
import numpy as np
import numpy.testing

import bcolz


@contextlib.contextmanager
def ctime(message=None):
    "Counts the time spent in some context"
    t = time.time()
    yield
    if message:
        print(message + ":\t", end="")
    print(round(time.time() - t, 4), "sec")


carootdir = "carraypickle.bcolz"
if os.path.exists(carootdir):
    shutil.rmtree(carootdir)
ctrootdir = "ctablepickle.bcolz"
if os.path.exists(ctrootdir):
    shutil.rmtree(ctrootdir)

N = int(1e7)
a = bcolz.arange(N, dtype="int32")
b = bcolz.arange(N, dtype="float32")
ca = bcolz.carray(a, rootdir=carootdir)
ct = bcolz.ctable([ca, b], names=['a', 'b'], rootdir=ctrootdir)

with ctime("Time spent pickling carray with N=%d" % N):
    s = pickle.dumps(ca)

with ctime("Time spent unpickling carray with N=%d" % N):
    ca2 = pickle.loads(s)

np.testing.assert_allclose(ca2[:], a)

with ctime("Time spent pickling ctable with N=%d" % N):
    s = pickle.dumps(ct)

with ctime("Time spent unpickling ctable with N=%d" % N):
    ct2 = pickle.loads(s)

np.testing.assert_allclose(ct2['a'][:], a)
np.testing.assert_allclose(ct2['b'][:], b)



from numpy cimport ndarray, dtype, npy_intp

cdef class chunk:
    cdef char typekind, isconstant
    cdef public int atomsize, itemsize, blocksize
    cdef public int nbytes, cbytes, cdbytes
    cdef int true_count
    cdef char *data
    cdef object atom, constant, dobject

    cdef void _getitem(self, int start, int stop, char *dest)
    cdef compress_data(self, char *data, size_t itemsize, size_t nbytes,
                       object cparams)
    cdef compress_arrdata(self, ndarray array, int itemsize,
                          object cparams, object _memory)


cdef class chunks(object):
    cdef object _rootdir, _mode
    cdef object dtype, cparams, lastchunkarr
    cdef object chunk_cached
    cdef npy_intp nchunks, nchunk_cached, len
    cdef int _iter_count

    cdef read_chunk(self, nchunk)
    cdef _save(self, nchunk, chunk_)
    cdef _chunk_file_name(self, nchunk)

cdef class carray:
    cdef public int itemsize, atomsize
    cdef int _chunksize, _chunklen, leftover
    cdef int nrowsinbuf, _row
    cdef int sss_mode, wheretrue_mode, where_mode
    cdef npy_intp startb, stopb
    cdef npy_intp start, stop, step, nextelement
    cdef npy_intp _nrow, nrowsread
    cdef npy_intp _nbytes, _cbytes
    cdef npy_intp nhits, limit, skip
    cdef npy_intp expectedlen
    cdef char *lastchunk
    cdef object lastchunkarr, where_arr, arr1
    cdef object _cparams, _dflt
    cdef object _dtype
    cdef object _safe
    cdef public object chunks
    cdef object _rootdir, datadir, metadir, _mode
    cdef object _attrs, iter_exhausted
    cdef ndarray iobuf, where_buf
    # For block cache
    cdef int idxcache
    cdef ndarray blockcache
    cdef char *datacache

    cdef void bool_update(self, boolarr, value)
    cdef int getitem_cache(self, npy_intp pos, char *dest)
    cdef reset_iter_sentinels(self)
    cdef int check_zeros(self, object barr)
    cdef _adapt_dtype(self, dtype_, shape)




# -*- coding: utf-8 -*-
########################################################################
#
#       License: BSD
#       Created: August 5, 2010
#       Author:  Francesc Alted - francesc@blosc.org
#
########################################################################

from __future__ import absolute_import

import os
import sys
import struct
import shutil
import textwrap
from bcolz.utils import to_ndarray

import numpy as np
from numpy.testing import assert_array_equal, assert_allclose
from bcolz.tests import common
from bcolz.tests.common import (
    MayBeDiskTest, TestCase, unittest, skipUnless, SkipTest)
import bcolz
from bcolz.py2help import xrange, PY2
from bcolz.carray_ext import chunk
from bcolz import carray
import pickle


import ctypes

is_64bit = (struct.calcsize("P") == 8)

if sys.version_info >= (3, 0):
    long = int


class initTest(TestCase):

    def test_roundtrip_from_transpose1(self):
        """Testing `__init__` called without `dtype` and a non-contiguous (transposed) array."""
        transposed_array = np.array([[0, 1, 2], [2, 1, 0]]).T
        assert_array_equal(transposed_array, carray(transposed_array, dtype=None))

    def test_roundtrip_from_transpose2(self):
        """Testing `__init__` called with `dtype` and a non-contiguous (transposed) array."""
        transposed_array = np.array([[0, 1, 2], [2, 1, 0]]).T
        assert_array_equal(transposed_array, carray(transposed_array, dtype=transposed_array.dtype))

    @unittest.skipIf(not bcolz.pandas_here, "cannot import pandas")
    def test_roundtrip_from_dataframe1(self):
        """Testing `__init__` called without `dtype` and a dataframe over non-contiguous data."""
        import pandas as pd
        df = pd.DataFrame(data={
            'a': np.arange(3),
            'b': np.arange(3)[::-1]
        })
        assert_array_equal(df, carray(df, dtype=None))

    @unittest.skipIf(not bcolz.pandas_here, "cannot import pandas")
    def test_roundtrip_from_dataframe2(self):
        """Testing `__init__` called with `dtype` and a dataframe over non-contiguous data."""
        import pandas as pd
        df = pd.DataFrame(data={
            'a': np.arange(3),
            'b': np.arange(3)[::-1]
        })
        ca = carray(df, dtype=np.dtype(np.float))
        assert_array_equal(df, ca)
        self.assertEqual(ca.dtype, np.dtype(np.float),
                         msg='carray has been created with invalid dtype')

    def test_dtype_None(self):
        """Testing `utils.to_ndarray` called without `dtype` and a non-contiguous (transposed) array."""
        array = np.array([[0, 1, 2], [2, 1, 0]]).T
        self.assertTrue(to_ndarray(array, None, safe=True).flags.contiguous,
                        msg='to_ndarray: Non contiguous arrays are not being consolidated when dtype is None')


class chunkTest(TestCase):

    def test01(self):
        """Testing `__getitem()__` method with scalars"""
        a = np.arange(1e3)
        b = chunk(a, atom=a.dtype, cparams=bcolz.cparams())
        # print "b[1]->", `b[1]`
        self.assertTrue(a[1] == b[1], "Values in key 1 are not equal")

    def test02(self):
        """Testing `__getitem()__` method with ranges"""
        a = np.arange(1e3)
        b = chunk(a, atom=a.dtype, cparams=bcolz.cparams())
        # print "b[1:3]->", `b[1:3]`
        assert_array_equal(a[1:3], b[1:3], "Arrays are not equal")

    def test03(self):
        """Testing `__getitem()__` method with ranges and steps"""
        a = np.arange(1e3)
        b = chunk(a, atom=a.dtype, cparams=bcolz.cparams())
        # print "b[1:8:3]->", `b[1:8:3]`
        assert_array_equal(a[1:8:3], b[1:8:3], "Arrays are not equal")

    def test04(self):
        """Testing `__getitem()__` method with long ranges"""
        a = np.arange(1e4)
        b = chunk(a, atom=a.dtype, cparams=bcolz.cparams())
        # print "b[1:8000]->", `b[1:8000]`
        assert_array_equal(a[1:8000], b[1:8000], "Arrays are not equal")


class pickleTest(MayBeDiskTest):

    disk = False

    def generate_data(self):
        return bcolz.arange(1e2)

    def test_pickleable(self):
        b = self.generate_data()
        s = pickle.dumps(b)
        if PY2:
            self.assertIsInstance(s, str)
        else:
            self.assertIsInstance(s, bytes)
        if self.disk:
            b2 = pickle.loads(s)
            # this should probably be self.assertEquals(b, b2)
            # but at the time == didn't work
            self.assertEquals(b2.rootdir, b.rootdir)


class pickleTestDisk(pickleTest):
    disk = True

    def generate_data(self):
        return bcolz.arange(1e2, rootdir=self.rootdir)


class getitemTest(MayBeDiskTest):

    def test01a(self):
        """Testing `__getitem()__` method with only a start"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(1)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test01b(self):
        """Testing `__getitem()__` method with only a (negative) start"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(-1)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test01c(self):
        """Testing `__getitem()__` method with only a (start,)"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        # print "b[(1,)]->", `b[(1,)]`
        self.assertTrue(a[(1,)] == b[(1,)],
                        "Values with key (1,) are not equal")

    def test01d(self):
        """Testing `__getitem()__` method with only a (large) start"""
        a = np.arange(1e4)
        b = bcolz.carray(a, rootdir=self.rootdir)
        sl = -2   # second last element
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test02a(self):
        """Testing `__getitem()__` method with ranges"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(1, 3)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test02b(self):
        """Testing `__getitem()__` method with ranges (negative start)"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(-3)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test02c(self):
        """Testing `__getitem()__` method with ranges (negative stop)"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(1, -3)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test02d(self):
        """Testing `__getitem()__` method with ranges (negative start, stop)"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(-3, -1)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test02e(self):
        """Testing `__getitem()__` method with start > stop"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(4, 3, 30)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test03a(self):
        """Testing `__getitem()__` method with ranges and steps (I)"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(1, 80, 3)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test03b(self):
        """Testing `__getitem()__` method with ranges and steps (II)"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(1, 80, 30)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test03c(self):
        """Testing `__getitem()__` method with ranges and steps (III)"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(990, 998, 2)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test03d(self):
        """Testing `__getitem()__` method with ranges and steps (IV)"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(4, 80, 3000)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test04a(self):
        """Testing `__getitem()__` method with lsmall ranges"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=16, rootdir=self.rootdir)
        sl = slice(1, 2)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test04ab(self):
        """Testing `__getitem()__` method with long ranges"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=100, rootdir=self.rootdir)
        sl = slice(1, 8000)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test04b(self):
        """Testing `__getitem()__` method with no start"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=100, rootdir=self.rootdir)
        sl = slice(None, 8000)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test04c(self):
        """Testing `__getitem()__` method with no stop"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=100, rootdir=self.rootdir)
        sl = slice(8000, None)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test04d(self):
        """Testing `__getitem()__` method with no start and no stop"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=100, rootdir=self.rootdir)
        sl = slice(None, None, 2)
        # print "b[sl]->", `b[sl]`
        assert_array_equal(a[sl], b[sl], "Arrays are not equal")

    def test05(self):
        """Testing `__getitem()__` method with negative steps"""
        a = np.arange(1e3)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        sl = slice(None, None, -3)
        # print "b[sl]->", `b[sl]`
        self.assertRaises(NotImplementedError, b.__getitem__, sl)


class getitemMemoryTest(getitemTest, TestCase):
    disk = False


class getitemDiskTest(getitemTest, TestCase):
    disk = True


class setitemTest(MayBeDiskTest):

    def test00a(self):
        """Testing `__setitem()__` method with only one element"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        b[1] = 10.
        a[1] = 10.
        # print "b->", `b`
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test00b(self):
        """Testing `__setitem()__` method with only one element (tuple)"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        b[(1,)] = 10.
        a[(1,)] = 10.
        # print "b->", `b`
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test01(self):
        """Testing `__setitem()__` method with a range"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        b[10:100] = np.arange(1e2 - 10.)
        a[10:100] = np.arange(1e2 - 10.)
        # print "b->", `b`
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test02(self):
        """Testing `__setitem()__` method with broadcasting"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        b[10:100] = 10.
        a[10:100] = 10.
        # print "b->", `b`
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test03(self):
        """Testing `__setitem()__` method with the complete range"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
        b[:] = np.arange(10., 1e2 + 10.)
        a[:] = np.arange(10., 1e2 + 10.)
        # print "b->", `b`
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test04a(self):
        """Testing `__setitem()__` method with start:stop:step"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=1, rootdir=self.rootdir)
        sl = slice(10, 100, 3)
        b[sl] = 10.
        a[sl] = 10.
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test04b(self):
        """Testing `__setitem()__` method with start:stop:step (II)"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=1, rootdir=self.rootdir)
        sl = slice(10, 11, 3)
        b[sl] = 10.
        a[sl] = 10.
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test04c(self):
        """Testing `__setitem()__` method with start:stop:step (III)"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=1, rootdir=self.rootdir)
        sl = slice(96, 100, 3)
        b[sl] = 10.
        a[sl] = 10.
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test04d(self):
        """Testing `__setitem()__` method with start:stop:step (IV)"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=1, rootdir=self.rootdir)
        sl = slice(2, 99, 30)
        b[sl] = 10.
        a[sl] = 10.
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(a, b[:], "__setitem__ not working correctly")

    def test05(self):
        """Testing `__setitem()__` method with negative step"""
        a = np.arange(1e2)
        b = bcolz.carray(a, chunklen=1, rootdir=self.rootdir)
        sl = slice(2, 99, -30)
        self.assertRaises(NotImplementedError, b.__setitem__, sl, 3.)


class setitemMemoryTest(setitemTest, TestCase):
    disk = False


class setitemDiskTest(setitemTest, TestCase):
    disk = True


class appendTest(MayBeDiskTest):

    def test00(self):
        """Testing `append()` method"""
        a = np.arange(1000)
        b = bcolz.carray(a, rootdir=self.rootdir)
        b.append(a)
        # print "b->", `b`
        c = np.concatenate((a, a))
        assert_array_equal(c, b[:], "Arrays are not equal")

    def test01(self):
        """Testing `append()` method (small chunklen)"""
        a = np.arange(1000)
        b = bcolz.carray(a, chunklen=1, rootdir=self.rootdir)
        b.append(a)
        # print "b->", `b`
        c = np.concatenate((a, a))
        assert_array_equal(c, b[:], "Arrays are not equal")

    def test02a(self):
        """Testing `append()` method (large chunklen I)"""
        a = np.arange(1000)
        b = bcolz.carray(a, chunklen=10*1000, rootdir=self.rootdir)
        b.append(a)
        # print "b->", `b`
        c = np.concatenate((a, a))
        assert_array_equal(c, b[:], "Arrays are not equal")

    def test02b(self):
        """Testing `append()` method (large chunklen II)"""
        a = np.arange(100*1000)
        b = bcolz.carray(a, chunklen=10*1000, rootdir=self.rootdir)
        b.append(a)
        # print "b->", `b`
        c = np.concatenate((a, a))
        assert_array_equal(c, b[:], "Arrays are not equal")

    def test02c(self):
        """Testing `append()` method (large chunklen III)"""
        a = np.arange(1000*1000)
        b = bcolz.carray(a, chunklen=100*1000-1, rootdir=self.rootdir)
        b.append(a)
        # print "b->", `b`
        c = np.concatenate((a, a))
        assert_array_equal(c, b[:], "Arrays are not equal")

    def test03(self):
        """Testing `append()` method (large append)"""
        a = np.arange(1e4)
        c = np.arange(2e5)
        b = bcolz.carray(a, rootdir=self.rootdir)
        b.append(c)
        # print "b->", `b`
        d = np.concatenate((a, c))
        assert_array_equal(d, b[:], "Arrays are not equal")


class appendMemoryTest(appendTest, TestCase):
    disk = False


class appendDiskTest(appendTest, TestCase):
    disk = True


class trimTest(MayBeDiskTest):

    def test00(self):
        """Testing `trim()` method"""
        b = bcolz.arange(1e3, rootdir=self.rootdir)
        b.trim(3)
        a = np.arange(1e3-3)
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")

    def test01(self):
        """Testing `trim()` method (small chunklen)"""
        b = bcolz.arange(1e2, chunklen=2, rootdir=self.rootdir)
        b.trim(5)
        a = np.arange(1e2-5)
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")

    def test02(self):
        """Testing `trim()` method (large trim)"""
        a = np.arange(2)
        b = bcolz.arange(1e4, rootdir=self.rootdir)
        b.trim(1e4-2)
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")

    def test03(self):
        """Testing `trim()` method (complete trim)"""
        a = np.arange(0.)
        b = bcolz.arange(1e4, rootdir=self.rootdir)
        b.trim(1e4)
        # print "b->", `b`
        self.assertTrue(len(a) == len(b), "Lengths are not equal")

    def test04(self):
        """Testing `trim()` method (trimming more than available items)"""
        a = np.arange(0.)
        b = bcolz.arange(1e4, rootdir=self.rootdir)
        # print "b->", `b`
        self.assertRaises(ValueError, b.trim, 1e4+1)

    def test05(self):
        """Testing `trim()` method (trimming zero items)"""
        a = np.arange(1e1)
        b = bcolz.arange(1e1, rootdir=self.rootdir)
        b.trim(0)
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")

    def test06(self):
        """Testing `trim()` method (negative number of items)"""
        a = np.arange(2e1)
        b = bcolz.arange(1e1, rootdir=self.rootdir)
        b.trim(-10)
        a[10:] = 0
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")


class trimMemoryTest(trimTest, TestCase):
    disk = False


class trimDiskTest(trimTest, TestCase):
    disk = True


class resizeTest(MayBeDiskTest):

    def test00a(self):
        """Testing `resize()` method (decrease)"""
        b = bcolz.arange(self.N, rootdir=self.rootdir)
        b.resize(self.N-3)
        a = np.arange(self.N-3)
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")

    def test00b(self):
        """Testing `resize()` method (increase)"""
        b = bcolz.arange(self.N, rootdir=self.rootdir)
        b.resize(self.N+3)
        a = np.arange(self.N+3)
        a[self.N:] = 0
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")

    def test01a(self):
        """Testing `resize()` method (decrease, large variation)"""
        b = bcolz.arange(self.N, rootdir=self.rootdir)
        b.resize(3)
        a = np.arange(3)
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")

    def test01b(self):
        """Testing `resize()` method (increase, large variation)"""
        b = bcolz.arange(self.N, dflt=1, rootdir=self.rootdir)
        b.resize(self.N*3)
        a = np.arange(self.N*3)
        a[self.N:] = 1
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")

    def test02(self):
        """Testing `resize()` method (zero size)"""
        b = bcolz.arange(self.N, rootdir=self.rootdir)
        b.resize(0)
        a = np.arange(0)
        # print "b->", `b`
        assert_array_equal(a, b[:], "Arrays are not equal")


class resize_smallTest(resizeTest, TestCase):
    N = 10


class resize_smallDiskTest(resizeTest, TestCase):
    N = 10
    disk = True


class resize_largeTest(resizeTest, TestCase):
    N = 10000


class resize_largeDiskTest(resizeTest, TestCase):
    N = 10000
    disk = True


class miscTest(MayBeDiskTest):

    def test00(self):
        """Testing __len__()"""
        a = np.arange(111)
        b = bcolz.carray(a, rootdir=self.rootdir)
        self.assertTrue(len(a) == len(b), "Arrays do not have the same length")

    def test01(self):
        """Testing __sizeof__() (big carrays)"""
        a = np.arange(2e5)
        b = bcolz.carray(a, rootdir=self.rootdir)
        # print "size b uncompressed-->", b.nbytes
        # print "size b compressed  -->", b.cbytes
        self.assertTrue(sys.getsizeof(b) < b.nbytes,
                        "carray does not seem to compress at all")

    def test02(self):
        """Testing __sizeof__() (small carrays)"""
        a = np.arange(111)
        b = bcolz.carray(a)
        # print "size b uncompressed-->", b.nbytes
        # print "size b compressed  -->", b.cbytes
        self.assertTrue(sys.getsizeof(b) > b.nbytes,
                        "carray compress too much??")


class miscMemoryTest(miscTest, TestCase):
    disk = False


class miscDiskTest(miscTest, TestCase):
    disk = True


class copyTest(MayBeDiskTest):

    N = int(1e5)

    def tearDown(self):
        # Restore defaults
        bcolz.cparams.setdefaults(clevel=5, shuffle=True, cname='blosclz')
        MayBeDiskTest.tearDown(self)

    def test00(self):
        """Testing copy() without params"""
        a = np.arange(111)
        b = bcolz.carray(a, rootdir=self.rootdir)
        c = b.copy()
        c.append(np.arange(111, 122))
        self.assertTrue(len(b) == 111, "copy() does not work well")
        self.assertTrue(len(c) == 122, "copy() does not work well")
        r = np.arange(122)
        assert_array_equal(c[:], r, "incorrect correct values after copy()")

    def test01(self):
        """Testing copy() with higher compression"""
        a = np.linspace(-1., 1., self.N)
        b = bcolz.carray(a, rootdir=self.rootdir)
        c = b.copy(cparams=bcolz.cparams(clevel=9))
        # print "b.cbytes, c.cbytes:", b.cbytes, c.cbytes
        self.assertTrue(b.cbytes > c.cbytes, "clevel not changed")

    def test02(self):
        """Testing copy() with lesser compression"""
        a = np.linspace(-1., 1., self.N)
        b = bcolz.carray(a, rootdir=self.rootdir)
        bcolz.cparams.setdefaults(clevel=1)
        c = b.copy()
        # print "b.cbytes, c.cbytes:", b.cbytes, c.cbytes
        self.assertTrue(b.cbytes < c.cbytes, "clevel not changed")

    def test03a(self):
        """Testing copy() with no shuffle"""
        a = np.linspace(-1., 1., self.N)
        b = bcolz.carray(a, rootdir=self.rootdir)
        bcolz.cparams.setdefaults(clevel=1)
        c = b.copy(cparams=bcolz.cparams(shuffle=False))
        # print "b.cbytes, c.cbytes:", b.cbytes, c.cbytes
        self.assertTrue(b.cbytes < c.cbytes, "shuffle not changed")

    def test03b(self):
        """Testing copy() with no shuffle (setdefaults version)"""
        a = np.linspace(-1., 1., self.N)
        b = bcolz.carray(a, rootdir=self.rootdir)
        bcolz.cparams.setdefaults(shuffle=False)
        c = b.copy()
        # print "b.cbytes, c.cbytes:", b.cbytes, c.cbytes
        self.assertTrue(b.cbytes < c.cbytes, "shuffle not changed")


class copyMemoryTest(copyTest, TestCase):
    disk = False


class copyDiskTest(copyTest, TestCase):
    disk = True


class viewTest(MayBeDiskTest):

    def tearDown(self):
        MayBeDiskTest.tearDown(self)

    def test00(self):
        """Testing view()"""
        a = np.arange(self.N)
        b = bcolz.carray(a, rootdir=self.rootdir)
        c = b.view()
        self.assertEqual(len(c), self.N)
        assert_array_equal(c[:], a)

    def test01(self):
        """Testing view() and appends"""
        a = np.arange(self.N)
        b = bcolz.carray(a, rootdir=self.rootdir)
        c = b.view()
        c.append(np.arange(self.N, self.N + 11))
        self.assertEqual(len(b), self.N)
        self.assertEqual(len(c), self.N + 11)
        r = np.arange(self.N + 11)
        assert_array_equal(b[:], a)
        assert_array_equal(c[:], r)

    def test02(self):
        """Testing view() and iterators"""
        a = np.arange(self.N, dtype='uint64')
        b = bcolz.carray(a, rootdir=self.rootdir)
        c = iter(b.view())
        u = c.iter(3)
        w = b.iter(2)
        self.assertEqual(sum(a[3:]), sum(u))
        self.assertEqual(sum(a[2:]), sum(w))


class small_viewMemoryTest(viewTest, TestCase):
    N = 111
    disk = False


class small_viewDiskTest(viewTest, TestCase):
    N = 111
    disk = True


class large_viewMemoryTest(viewTest, TestCase):
    N = int(1e5)
    disk = False


class large_viewDiskTest(viewTest, TestCase):
    N = int(1e5)
    disk = True


class iterTest(MayBeDiskTest):

    def test00a(self):
        """Testing `next()` method"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        for i in range(101):
            self.assertEqual(i, next(b))

    def test00b(self):
        """Testing `iter()` method"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        # print "sum iter1->", sum(b)
        # print "sum iter2->", sum((v for v in b))
        self.assertEqual(sum(a), sum(b))
        self.assertEqual(sum((v for v in a)), sum((v for v in b)))

    def test01a(self):
        """Testing `iter()` method with a positive start"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        # print "sum iter->", sum(b.iter(3))
        self.assertTrue(sum(a[3:]) == sum(b.iter(3)), "Sums are not equal")

    def test01b(self):
        """Testing `iter()` method with a negative start"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        # print "sum iter->", sum(b.iter(-3))
        self.assertTrue(sum(a[-3:]) == sum(b.iter(-3)), "Sums are not equal")

    def test02a(self):
        """Testing `iter()` method with positive start, stop"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        # print "sum iter->", sum(b.iter(3, 24))
        self.assertTrue(sum(a[3:24]) == sum(b.iter(3, 24)),
                        "Sums are not equal")

    def test02b(self):
        """Testing `iter()` method with negative start, stop"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        # print "sum iter->", sum(b.iter(-24, -3))
        self.assertTrue(sum(a[-24:-3]) == sum(b.iter(-24, -3)),
                        "Sums are not equal")

    def test02c(self):
        """Testing `iter()` method with positive start, negative stop"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        # print "sum iter->", sum(b.iter(24, -3))
        self.assertTrue(sum(a[24:-3]) == sum(b.iter(24, -3)),
                        "Sums are not equal")

    def test03a(self):
        """Testing `iter()` method with only step"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        # print "sum iter->", sum(b.iter(step=4))
        self.assertTrue(sum(a[::4]) == sum(b.iter(step=4)),
                        "Sums are not equal")

    def test03b(self):
        """Testing `iter()` method with start, stop, step"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        # print "sum iter->", sum(b.iter(3, 24, 4))
        self.assertTrue(sum(a[3:24:4]) == sum(b.iter(3, 24, 4)),
                        "Sums are not equal")

    def test03c(self):
        """Testing `iter()` method with negative step"""
        a = np.arange(101)
        b = bcolz.carray(a, chunklen=2, rootdir=self.rootdir)
        self.assertRaises(NotImplementedError, b.iter, 0, 1, -3)

    def test04(self):
        """Testing `iter()` method with large zero arrays"""
        a = np.zeros(int(1e4), dtype='f8')
        b = bcolz.carray(a, chunklen=100, rootdir=self.rootdir)
        c = bcolz.fromiter((v for v in b), dtype='f8', count=len(a))
        # print "c ->", repr(c)
        assert_allclose(a, c[:], err_msg="iterator fails on zeros")

    def test05(self):
        """Testing `iter()` method with `limit`"""
        a = np.arange(1e4, dtype='f8')
        b = bcolz.carray(a, chunklen=100, rootdir=self.rootdir)
        c = bcolz.fromiter((v for v in b.iter(limit=1010)), dtype='f8',
                           count=1010)
        # print "c ->", repr(c)
        assert_allclose(a[:1010], c, err_msg="iterator fails on zeros")

    def test06(self):
        """Testing `iter()` method with `skip`"""
        a = np.arange(1e4, dtype='f8')
        b = bcolz.carray(a, chunklen=100, rootdir=self.rootdir)
        c = bcolz.fromiter((v for v in b.iter(skip=1010)), dtype='f8',
                           count=10000-1010)
        # print "c ->", repr(c)
        assert_allclose(a[1010:], c, err_msg="iterator fails on zeros")

    def test07(self):
        """Testing `iter()` method with `limit` and `skip`"""
        a = np.arange(1e4, dtype='f8')
        b = bcolz.carray(a, chunklen=100, rootdir=self.rootdir)
        c = bcolz.fromiter((v for v in b.iter(limit=1010, skip=1010)),
                           dtype='f8',
                           count=1010)
        # print "c ->", repr(c)
        assert_allclose(a[1010:2020], c, err_msg="iterator fails on zeros")

    def test08a(self):
        """Testing several iterators in stage (I)"""
        a = np.arange(1e3)
        b = bcolz.carray(a, rootdir=self.rootdir)
        u = iter(b)
        w = b.iter(2, 20, 2)
        self.assertEqual(a.tolist(), list(b))
        self.assertEqual(sum(a), sum(u))
        self.assertEqual(sum(a[2:20:2]), sum(w))

    def test08b(self):
        """Testing several iterators in stage (II)"""
        a = np.arange(1e3)
        b = bcolz.carray(a, rootdir=self.rootdir)
        u = b.iter(3, 30, 3)
        w = b.iter(2, 20, 2)
        self.assertEqual(a.tolist(), list(b))
        self.assertEqual(sum(a[3:30:3]), sum(u))
        self.assertEqual(sum(a[2:20:2]), sum(w))

    def test09a(self):
        """Testing several iterators in parallel (I)"""
        a = np.arange(10)
        b = bcolz.carray(a, rootdir=self.rootdir)
        b1 = iter(b)
        b2 = iter(b)
        a1 = iter(a)
        a2 = iter(a)
        # print "result:",  [i for i in zip(b1, b2)]
        self.assertEqual([i for i in zip(a1, a2)], [i for i in zip(b1, b2)])

    def test09b(self):
        """Testing several iterators in parallel (II)"""
        a = np.arange(10)
        b = bcolz.carray(a, rootdir=self.rootdir)
        b1 = b.iter(2, 10, 2)
        b2 = b.iter(1, 5, 1)
        a1 = iter(a[2:10:2])
        a2 = iter(a[1:5:1])
        # print "result:",  [i for i in zip(b1, b2)]
        self.assertEqual([i for i in zip(a1, a2)], [i for i in zip(b1, b2)])

    def test10a(self):
        """Testing the reuse of exhausted iterators (I)"""
        a = np.arange(10)
        b = bcolz.carray(a, rootdir=self.rootdir)
        bi = iter(b)
        ai = iter(a)
        self.assertEqual([i for i in ai], [i for i in bi])
        self.assertEqual([i for i in ai], [i for i in bi])

    def test10b(self):
        """Testing the reuse of exhausted iterators (II)"""
        a = np.arange(10)
        b = bcolz.carray(a, rootdir=self.rootdir)
        bi = b.iter(2, 10, 2)
        ai = iter(a[2:10:2])
        # print "result:", [i for i in bi]
        self.assertEqual([i for i in ai], [i for i in bi])
        self.assertEqual([i for i in ai], [i for i in bi])


class iterMemoryTest(iterTest, TestCase):
    disk = False


class iterDiskTest(iterTest, TestCase):
    disk = True


class iterblocksTest(MayBeDiskTest):

    def test00(self):
        """Testing `iterblocks` method with no blen, no start, no stop"""
        N = self.N
        a = bcolz.fromiter(xrange(N), dtype=np.float64, count=N,
                           rootdir=self.rootdir)
        l, s = 0, 0
        for block in bcolz.iterblocks(a):
            l += len(block)
            s += block.sum()
        self.assertEqual(l, N)
        # as per Gauss summation formula
        self.assertEqual(s, (N - 1) * (N / 2))

    def test01(self):
        """Testing `iterblocks` method with no start, no stop"""
        N, blen = self.N, 100
        a = bcolz.fromiter(xrange(N), dtype=np.float64, count=N,
                           rootdir=self.rootdir)
        l, s = 0, 0
        for block in bcolz.iterblocks(a, blen):
            if l == 0:
                self.assertEqual(len(block), blen)
            l += len(block)
            s += block.sum()
        self.assertEqual(l, N)

    def test02(self):
        """Testing `iterblocks` method with no stop"""
        N, blen = self.N, 100
        a = bcolz.fromiter(xrange(N), dtype=np.float64, count=N,
                           rootdir=self.rootdir)
        l, s = 0, 0
        for block in bcolz.iterblocks(a, blen, blen-1):
            l += len(block)
            s += block.sum()
        self.assertEqual(l, (N - (blen - 1)))
        self.assertEqual(s, np.arange(blen-1, N).sum())

    def test03(self):
        """Testing `iterblocks` method with all parameters set"""
        N, blen = self.N, 100
        a = bcolz.fromiter(xrange(N), dtype=np.float64, count=N,
                           rootdir=self.rootdir)
        l, s = 0, 0
        for block in bcolz.iterblocks(a, blen, blen-1, 3*blen+2):
            l += len(block)
            s += block.sum()
        mlen = min(N - (blen - 1), 2*blen + 3)
        self.assertEqual(l, mlen)
        slen = min(N, 3*blen + 2)
        self.assertEqual(s, np.arange(blen-1, slen).sum())


class small_iterblocksMemoryTest(iterblocksTest, TestCase):
    N = 120
    disk = False


class small_iterblocksDiskTest(iterblocksTest, TestCase):
    N = 120
    disk = True


class large_iterblocksMemoryTest(iterblocksTest, TestCase):
    N = 10000
    disk = False


class large_iterblocksDiskTest(iterblocksTest, TestCase):
    N = 10030
    disk = True


class wheretrueTest(TestCase):

    def test00(self):
        """Testing `wheretrue()` iterator (all true values)"""
        a = np.arange(1, 11) > 0
        b = bcolz.carray(a)
        wt = a.nonzero()[0].tolist()
        cwt = [i for i in b.wheretrue()]
        # print "numpy ->", a.nonzero()[0].tolist()
        # print "where ->", [i for i in b.wheretrue()]
        self.assertTrue(wt == cwt, "wheretrue() does not work correctly")

    def test01(self):
        """Testing `wheretrue()` iterator (all false values)"""
        a = np.arange(1, 11) < 0
        b = bcolz.carray(a)
        wt = a.nonzero()[0].tolist()
        cwt = [i for i in b.wheretrue()]
        # print "numpy ->", a.nonzero()[0].tolist()
        # print "where ->", [i for i in b.wheretrue()]
        self.assertTrue(wt == cwt, "wheretrue() does not work correctly")

    def test02(self):
        """Testing `wheretrue()` iterator (all false values, large array)"""
        a = np.arange(1, 1e5) < 0
        b = bcolz.carray(a)
        wt = a.nonzero()[0].tolist()
        cwt = [i for i in b.wheretrue()]
        # print "numpy ->", a.nonzero()[0].tolist()
        # print "where ->", [i for i in b.wheretrue()]
        self.assertTrue(wt == cwt, "wheretrue() does not work correctly")

    def test03(self):
        """Testing `wheretrue()` iterator (mix of true/false values)"""
        a = np.arange(1, 11) > 5
        b = bcolz.carray(a)
        wt = a.nonzero()[0].tolist()
        cwt = [i for i in b.wheretrue()]
        # print "numpy ->", a.nonzero()[0].tolist()
        # print "where ->", [i for i in b.wheretrue()]
        self.assertTrue(wt == cwt, "wheretrue() does not work correctly")

    def test04(self):
        """Testing `wheretrue()` iterator with `limit`"""
        a = np.arange(1, 11) > 5
        b = bcolz.carray(a)
        wt = a.nonzero()[0].tolist()[:3]
        cwt = [i for i in b.wheretrue(limit=3)]
        # print "numpy ->", a.nonzero()[0].tolist()[:3]
        # print "where ->", [i for i in b.wheretrue(limit=3)]
        self.assertTrue(wt == cwt, "wheretrue() does not work correctly")

    def test05(self):
        """Testing `wheretrue()` iterator with `skip`"""
        a = np.arange(1, 11) > 5
        b = bcolz.carray(a)
        wt = a.nonzero()[0].tolist()[2:]
        cwt = [i for i in b.wheretrue(skip=2)]
        # print "numpy ->", a.nonzero()[0].tolist()[2:]
        # print "where ->", [i for i in b.wheretrue(skip=2)]
        self.assertTrue(wt == cwt, "wheretrue() does not work correctly")

    def test06(self):
        """Testing `wheretrue()` iterator with `limit` and `skip`"""
        a = np.arange(1, 11) > 5
        b = bcolz.carray(a)
        wt = a.nonzero()[0].tolist()[2:4]
        cwt = [i for i in b.wheretrue(skip=2, limit=2)]
        # print "numpy ->", a.nonzero()[0].tolist()[2:4]
        # print "where ->", [i for i in b.wheretrue(limit=2,skip=2)]
        self.assertTrue(wt == cwt, "wheretrue() does not work correctly")

    def test07(self):
        """Testing `wheretrue()` iterator with `limit` and `skip` (zeros)"""
        a = np.arange(10000) > 5000
        b = bcolz.carray(a, chunklen=100)
        wt = a.nonzero()[0].tolist()[1020:2040]
        cwt = [i for i in b.wheretrue(skip=1020, limit=1020)]
        # print "numpy ->", a.nonzero()[0].tolist()[1020:2040]
        # print "where ->", [i for i in b.wheretrue(limit=1020,skip=1020)]
        self.assertTrue(wt == cwt, "wheretrue() does not work correctly")

    def test08(self):
        """Testing several iterators in stage"""
        a = np.arange(10000) > 5000
        b = bcolz.carray(a, chunklen=100)
        u = b.wheretrue(skip=1020, limit=1020)
        w = b.wheretrue(skip=1030, limit=1030)
        self.assertEqual(a.nonzero()[0].tolist()[1020:2040], list(u))
        self.assertEqual(a.nonzero()[0].tolist()[1030:2060], list(w))

    def test09(self):
        """Testing several iterators in parallel"""
        a = np.arange(10000) > 5000
        b = bcolz.carray(a, chunklen=100)
        b1 = b.wheretrue(skip=1020, limit=1020)
        b2 = b.wheretrue(skip=1030, limit=1020)
        a1 = a.nonzero()[0].tolist()[1020:2040]
        a2 = a.nonzero()[0].tolist()[1030:2050]
        # print "result:",  [i for i in zip(b1, b2)]
        self.assertEqual([i for i in zip(a1, a2)], [i for i in zip(b1, b2)])

    def test10(self):
        """Testing the reuse of exhausted iterators"""
        a = np.arange(10000) > 5000
        b = bcolz.carray(a, chunklen=100)
        bi = b.wheretrue(skip=1020, limit=1020)
        ai = iter(np.array(a.nonzero()[0].tolist()[1020:2040]))
        self.assertEqual([i for i in ai], [i for i in bi])
        self.assertEqual([i for i in ai], [i for i in bi])


class whereTest(TestCase):

    def test00(self):
        """Testing `where()` iterator (all true values)"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v > 0]
        cwt = [v for v in b.where(a > 0)]
        # print "numpy ->", [v for v in a if v>0]
        # print "where ->", [v for v in b.where(a>0)]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test01(self):
        """Testing `where()` iterator (all false values)"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v < 0]
        cwt = [v for v in b.where(a < 0)]
        # print "numpy ->", [v for v in a if v<0]
        # print "where ->", [v for v in b.where(a<0)]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test02a(self):
        """Testing `where()` iterator (mix of true/false values, I)"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v <= 5]
        cwt = [v for v in b.where(a <= 5)]
        # print "numpy ->", [v for v in a if v<=5]
        # print "where ->", [v for v in b.where(a<=5)]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test02b(self):
        """Testing `where()` iterator (mix of true/false values, II)"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v <= 5 and v > 2]
        cwt = [v for v in b.where((a <= 5) & (a > 2))]
        # print "numpy ->", [v for v in a if v<=5 and v>2]
        # print "where ->", [v for v in b.where((a<=5) & (a>2))]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test02c(self):
        """Testing `where()` iterator (mix of true/false values, III)"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v <= 5 or v > 8]
        cwt = [v for v in b.where((a <= 5) | (a > 8))]
        # print "numpy ->", [v for v in a if v<=5 or v>8]
        # print "where ->", [v for v in b.where((a<=5) | (a>8))]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test03(self):
        """Testing `where()` iterator (using a boolean carray)"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v <= 5]
        cwt = [v for v in b.where(bcolz.carray(a <= 5))]
        # print "numpy ->", [v for v in a if v<=5]
        # print "where ->", [v for v in b.where(bcolz.carray(a<=5))]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test04(self):
        """Testing `where()` iterator using `limit`"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v <= 5][:3]
        cwt = [v for v in b.where(bcolz.carray(a <= 5), limit=3)]
        # print "numpy ->", [v for v in a if v<=5][:3]
        # print "where ->", [v for v in b.where(bcolz.carray(a<=5), limit=3)]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test05(self):
        """Testing `where()` iterator using `skip`"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v <= 5][2:]
        cwt = [v for v in b.where(bcolz.carray(a <= 5), skip=2)]
        # print "numpy ->", [v for v in a if v<=5][2:]
        # print "where ->", [v for v in b.where(bcolz.carray(a<=5), skip=2)]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test06(self):
        """Testing `where()` iterator using `limit` and `skip`"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        wt = [v for v in a if v <= 5][1:4]
        cwt = [v for v in b.where(bcolz.carray(a <= 5), limit=3, skip=1)]
        # print "numpy ->", [v for v in a if v<=5][1:4]
        # print "where ->", [v for v in b.where(bcolz.carray(a<=5),
        #                                      limit=3, skip=1)]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test07(self):
        """Testing `where()` iterator using `limit` and `skip` (zeros)"""
        a = np.arange(10000)
        b = bcolz.carray(a,)
        wt = [v for v in a if v <= 5000][1010:2020]
        cwt = [v for v in b.where(bcolz.carray(a <= 5000, chunklen=100),
                                  limit=1010, skip=1010)]
        # print "numpy ->", [v for v in a if v>=5000][1010:2020]
        # print "where ->", [v for v in b.where(bcolz.carray(a>=5000,
        #                                                    chunklen=100),
        #                                       limit=1010, skip=1010)]
        self.assertTrue(wt == cwt, "where() does not work correctly")

    def test08(self):
        """Testing several iterators in stage"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        ul = [v for v in a if v <= 5]
        u = b.where(a <= 5)
        wl = [v for v in a if v <= 6]
        w = b.where(a <= 6)
        self.assertEqual(ul, list(u))
        self.assertEqual(wl, list(w))

    def test09(self):
        """Testing several iterators in parallel"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        b1 = b.where(a <= 5)
        b2 = b.where(a <= 6)
        a1 = [v for v in a if v <= 5]
        a2 = [v for v in a if v <= 6]
        # print "result:",  [i for i in zip(b1, b2)]
        self.assertEqual([i for i in zip(a1, a2)], [i for i in zip(b1, b2)])

    def test10(self):
        """Testing the reuse of exhausted iterators"""
        a = np.arange(1, 11)
        b = bcolz.carray(a)
        bi = b.where(a <= 5)
        ai = (v for v in a if v <= 5)
        self.assertEqual([i for i in ai], [i for i in bi])
        self.assertEqual([i for i in ai], [i for i in bi])


class fancy_indexing_getitemTest(TestCase):

    def test00(self):
        """Testing fancy indexing (short list)"""
        a = np.arange(1, 111)
        b = bcolz.carray(a)
        c = b[[3, 1]]
        r = a[[3, 1]]
        assert_array_equal(c, r, "fancy indexing does not work correctly")

    def test01(self):
        """Testing fancy indexing (large list, numpy)"""
        a = np.arange(1, 1e4)
        b = bcolz.carray(a)
        idx = np.random.randint(1000, size=1000)
        c = b[idx]
        r = a[idx]
        assert_array_equal(c, r, "fancy indexing does not work correctly")

    def test02(self):
        """Testing fancy indexing (empty list)"""
        a = np.arange(101)
        b = bcolz.carray(a)
        c = b[[]]
        r = a[[]]
        assert_array_equal(c, r, "fancy indexing does not work correctly")

    def test03(self):
        """Testing fancy indexing (list of floats)"""
        a = np.arange(1, 101)
        b = bcolz.carray(a)
        c = b[[1.1, 3.3]]
        r = a[[1.1, 3.3]]
        assert_array_equal(c, r, "fancy indexing does not work correctly")

    def test04(self):
        """Testing fancy indexing (list of floats, numpy)"""
        a = np.arange(1, 101)
        b = bcolz.carray(a)
        idx = np.array([1.1, 3.3], dtype='f8')
        self.assertRaises(IndexError, b.__getitem__, idx)

    def test05(self):
        """Testing `where()` iterator (using bool in fancy indexing)"""
        a = np.arange(1, 110)
        b = bcolz.carray(a, chunklen=10)
        wt = a[a < 5]
        cwt = b[a < 5]
        # print "numpy ->", a[a<5]
        # print "where ->", b[a<5]
        assert_array_equal(wt, cwt, "where() does not work correctly")

    def test06(self):
        """Testing `where()` iterator (using carray bool in fancy indexing)"""
        a = np.arange(1, 110)
        b = bcolz.carray(a, chunklen=10)
        wt = a[(a < 5) | (a > 9)]
        cwt = b[bcolz.carray((a < 5) | (a > 9))]
        # print "numpy ->", a[(a<5)|(a>9)]
        # print "where ->", b[bcolz.carray((a<5)|(a>9))]
        assert_array_equal(wt, cwt, "where() does not work correctly")


class fancy_indexing_setitemTest(TestCase):

    def test00(self):
        """Testing fancy indexing with __setitem__ (small values)"""
        a = np.arange(1, 111)
        b = bcolz.carray(a, chunklen=10)
        sl = [3, 1]
        b[sl] = (10, 20)
        a[sl] = (10, 20)
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(b[:], a, "fancy indexing does not work correctly")

    def test01(self):
        """Testing fancy indexing with __setitem__ (large values)"""
        a = np.arange(1, 1e3)
        b = bcolz.carray(a, chunklen=10)
        sl = [0, 300, 998]
        b[sl] = (5, 10, 20)
        a[sl] = (5, 10, 20)
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(b[:], a, "fancy indexing does not work correctly")

    def test02(self):
        """Testing fancy indexing with __setitem__ (large list)"""
        a = np.arange(0, 1000)
        b = bcolz.carray(a, chunklen=10)
        sl = np.random.randint(0, 1000, size=3*30)
        vals = np.random.randint(1, 1000, size=3*30)
        b[sl] = vals
        a[sl] = vals
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(b[:], a, "fancy indexing does not work correctly")

    def test03(self):
        """Testing fancy indexing with __setitem__ (bool array)"""
        a = np.arange(1, 1e2)
        b = bcolz.carray(a, chunklen=10)
        sl = a > 5
        b[sl] = 3.
        a[sl] = 3.
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(b[:], a, "fancy indexing does not work correctly")

    def test04(self):
        """Testing fancy indexing with __setitem__ (bool carray)"""
        a = np.arange(1, 1e2)
        b = bcolz.carray(a, chunklen=10)
        bc = (a > 5) & (a < 40)
        sl = bcolz.carray(bc)
        b[sl] = 3.
        a[bc] = 3.
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(b[:], a, "fancy indexing does not work correctly")

    def test05(self):
        """Testing fancy indexing with __setitem__ (bool, value not scalar)"""
        a = np.arange(1, 1e2)
        b = bcolz.carray(a, chunklen=10)
        sl = a < 5
        b[sl] = range(6, 10)
        a[sl] = range(6, 10)
        # print "b[%s] -> %r" % (sl, b)
        assert_array_equal(b[:], a, "fancy indexing does not work correctly")


class fromiterTest(TestCase):

    def test00(self):
        """Testing fromiter (short iter)"""
        a = np.arange(1, 111)
        b = bcolz.fromiter(iter(a), dtype='i4', count=len(a))
        assert_array_equal(b[:], a, "fromiter does not work correctly")

    def test01a(self):
        """Testing fromiter (long iter)"""
        N = 1e4
        a = (i for i in xrange(int(N)))
        b = bcolz.fromiter(a, dtype='f8', count=int(N))
        c = np.arange(N)
        assert_array_equal(b[:], c, "fromiter does not work correctly")

    def test01b(self):
        """Testing fromiter (long iter, chunk is multiple of iter length)"""
        N = 1e4
        a = (i for i in xrange(int(N)))
        b = bcolz.fromiter(a, dtype='f8', chunklen=1000, count=int(N))
        c = np.arange(N)
        assert_array_equal(b[:], c, "fromiter does not work correctly")

    def test02(self):
        """Testing fromiter (empty iter)"""
        a = np.array([], dtype="f8")
        b = bcolz.fromiter(iter(a), dtype='f8', count=-1)
        assert_array_equal(b[:], a, "fromiter does not work correctly")

    def test03(self):
        """Testing fromiter (dtype conversion)"""
        a = np.arange(101, dtype="f8")
        b = bcolz.fromiter(iter(a), dtype='f4', count=len(a))
        assert_array_equal(b[:], a, "fromiter does not work correctly")

    def test04a(self):
        """Testing fromiter method with large iterator"""
        N = 10*1000
        a = np.fromiter((i*2 for i in xrange(N)), dtype='f8')
        b = bcolz.fromiter((i*2 for i in xrange(N)), dtype='f8', count=len(a))
        assert_array_equal(b[:], a, "iterator with a hint fails")

    def test04b(self):
        """Testing fromiter method with large iterator with a hint"""
        N = 10*1000
        a = np.fromiter((i*2 for i in xrange(N)), dtype='f8', count=N)
        b = bcolz.fromiter((i*2 for i in xrange(N)), dtype='f8', count=N)
        assert_array_equal(b[:], a, "iterator with a hint fails")


class evalTest(MayBeDiskTest):

    vm = "python"

    def setUp(self):
        self.prev_vm = bcolz.defaults.eval_vm
        if bcolz.numexpr_here:
            bcolz.defaults.eval_vm = self.vm
        else:
            bcolz.defaults.eval_vm = "python"
        MayBeDiskTest.setUp(self)

    def tearDown(self):
        bcolz.defaults.eval_vm = self.prev_vm
        MayBeDiskTest.tearDown(self)

    def test00(self):
        """Testing eval() with only scalars and constants"""
        a = 3
        cr = bcolz.eval("2 * a", rootdir=self.rootdir)
        # print "bcolz.eval ->", cr
        self.assertTrue(cr == 6, "eval does not work correctly")

    def test01(self):
        """Testing eval() with only carrays"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        if self.rootdir:
            dirc, dird = self.rootdir+'.c', self.rootdir+'.d'
        else:
            dirc, dird = None, None
        c = bcolz.carray(a, rootdir=dirc)
        d = bcolz.carray(b, rootdir=dird)
        cr = bcolz.eval("c * d")
        nr = a * b
        # print "bcolz.eval ->", cr
        # print "numpy   ->", nr
        assert_array_equal(cr[:], nr, "eval does not work correctly")

    def test02(self):
        """Testing eval() with only ndarrays"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        cr = bcolz.eval("a * b", rootdir=self.rootdir)
        nr = a * b
        # print "bcolz.eval ->", cr
        # print "numpy   ->", nr
        assert_array_equal(cr[:], nr, "eval does not work correctly")

    def test03(self):
        """Testing eval() with a mix of carrays and ndarrays"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        if self.rootdir:
            dirc, dird = self.rootdir+'.c', self.rootdir+'.d'
        else:
            dirc, dird = None, None
        c = bcolz.carray(a, rootdir=dirc)
        d = bcolz.carray(b, rootdir=dird)
        cr = bcolz.eval("a * d")
        nr = a * b
        # print "bcolz.eval ->", cr
        # print "numpy   ->", nr
        assert_array_equal(cr[:], nr, "eval does not work correctly")

    def test04(self):
        """Testing eval() with a mix of carray, ndarray and scalars"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        if self.rootdir:
            dirc, dird = self.rootdir+'.c', self.rootdir+'.d'
        else:
            dirc, dird = None, None
        c = bcolz.carray(a, rootdir=dirc)
        d = bcolz.carray(b, rootdir=dird)
        cr = bcolz.eval("a + 2 * d - 3")
        nr = a + 2 * b - 3
        # print "bcolz.eval ->", cr
        # print "numpy   ->", nr
        assert_array_equal(cr[:], nr, "eval does not work correctly")

    def test05(self):
        """Testing eval() with a mix of carray, ndarray and scalars"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        c, d = bcolz.carray(a, rootdir=self.rootdir), b
        cr = bcolz.eval("a + 2 * d - 3")
        nr = a + 2 * b - 3
        # print "bcolz.eval ->", cr
        # print "numpy   ->", nr
        assert_array_equal(cr[:], nr, "eval does not work correctly")

    def test06(self):
        """Testing eval() with only scalars and arrays"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        c, d = bcolz.carray(a, rootdir=self.rootdir), b
        cr = bcolz.eval("d - 3")
        nr = b - 3
        # print "bcolz.eval ->", cr
        # print "numpy   ->", nr
        assert_array_equal(cr[:], nr, "eval does not work correctly")

    def test07(self):
        """Testing eval() via expression on __getitem__"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        c, d = bcolz.carray(a, rootdir=self.rootdir), b
        cr = c["a + 2 * d - 3 > 0"]
        nr = a[(a + 2 * b - 3) > 0]
        # print "ca[expr] ->", cr
        # print "numpy   ->", nr
        assert_array_equal(cr[:], nr, "carray[expr] does not work correctly")

    def test08(self):
        """Testing eval() via expression with lists (raise ValueError)"""
        a, b = range(int(self.N)), range(int(self.N))
        depth = 3
        if sys.version_info >= (3, 0):
            depth += 1  # curiously enough, Python 3 needs one level more
        self.assertRaises(ValueError, bcolz.eval, "a*3", depth=depth,
                          rootdir=self.rootdir)
        self.assertRaises(ValueError, bcolz.eval, "b*3", depth=depth,
                          rootdir=self.rootdir)

    def test09(self):
        """Testing eval() via expression on __setitem__ (I)"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        c, d = bcolz.carray(a, rootdir=self.rootdir), b
        c["a + 2 * d - 3 > 0"] = 3
        a[(a + 2 * b - 3) > 0] = 3
        # print "carray ->", c
        # print "numpy  ->", a
        assert_array_equal(c[:], a, "carray[expr] = v does not work correctly")

    def test10(self):
        """Testing eval() via expression on __setitem__ (II)"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        c, d = bcolz.carray(a, rootdir=self.rootdir), b
        c["a + 2 * d - 3 > 1000"] = 0
        a[(a + 2 * b - 3) > 1000] = 0
        # print "carray ->", c
        # print "numpy  ->", a
        assert_array_equal(c[:], a, "carray[expr] = v does not work correctly")

    def test11(self):
        """Testing eval() with functions like `np.sin()`"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        c, d = bcolz.carray(a, rootdir=self.rootdir), bcolz.carray(b)
        if self.vm == "python":
            cr = bcolz.eval("np.sin(c) + 2 * np.log(d) - 3")
        else:
            cr = bcolz.eval("sin(c) + 2 * log(d) - 3")
        nr = np.sin(a) + 2 * np.log(b) - 3
        # print "bcolz.eval ->", cr
        # print "numpy   ->", nr
        assert_array_equal(cr[:], nr, "eval does not work correctly")

    def test12(self):
        """Testing eval() with `out_flavor` == 'numpy'"""
        a, b = np.arange(self.N), np.arange(1, self.N+1)
        c, d = bcolz.carray(a), bcolz.carray(b, rootdir=self.rootdir)
        cr = bcolz.eval("c + 2 * d - 3", out_flavor='numpy')
        nr = a + 2 * b - 3
        # print "bcolz.eval ->", cr, type(cr)
        # print "numpy   ->", nr
        self.assertTrue(type(cr) == np.ndarray)
        assert_array_equal(cr, nr, "eval does not work correctly")


class evalSmall(evalTest):
    N = 10


class evalDiskSmall(evalTest):
    N = 10
    disk = True


class evalBig(evalTest):
    N = 1e4


class evalDiskBig(evalTest):
    N = 1e4
    disk = True


class evalSmallNE(evalTest):
    N = 10
    vm = "numexpr"


class evalDiskSmallNE(evalTest):
    N = 10
    vm = "numexpr"
    disk = True


class evalBigNE(evalTest):
    N = 1e4
    vm = "numexpr"


class evalDiskBigNE(evalTest):
    N = 1e4
    vm = "numexpr"
    disk = True


class computeMethodsTest(TestCase):

    def test00(self):
        """Testing sum()."""
        a = np.arange(1e5)
        sa = a.sum()
        ac = bcolz.carray(a)
        sac = ac.sum()
        # print "numpy sum-->", sa
        # print "carray sum-->", sac
        self.assertTrue(sa.dtype == sac.dtype,
                        "sum() is not working correctly.")
        self.assertTrue(sa == sac, "sum() is not working correctly.")

    def test01(self):
        """Testing sum() with dtype."""
        a = np.arange(1e5)
        sa = a.sum(dtype='i8')
        ac = bcolz.carray(a)
        sac = ac.sum(dtype='i8')
        # print "numpy sum-->", sa
        # print "carray sum-->", sac
        self.assertTrue(sa.dtype == sac.dtype,
                        "sum() is not working correctly.")
        self.assertTrue(sa == sac, "sum() is not working correctly.")

    def test02(self):
        """Testing sum() with strings (TypeError)."""
        ac = bcolz.zeros(10, 'S3')
        self.assertRaises(TypeError, ac.sum)


class arangeTest(MayBeDiskTest):

    def test00(self):
        """Testing arange() with only a `stop`."""
        a = np.arange(self.N)
        ac = bcolz.arange(self.N, rootdir=self.rootdir)
        self.assertTrue(np.all(a == ac))

    def test01(self):
        """Testing arange() with a `start` and `stop`."""
        a = np.arange(3, self.N)
        ac = bcolz.arange(3, self.N, rootdir=self.rootdir)
        self.assertTrue(np.all(a == ac))

    def test02(self):
        """Testing arange() with a `start`, `stop` and `step`."""
        a = np.arange(3, self.N, 4)
        ac = bcolz.arange(3, self.N, 4, rootdir=self.rootdir)
        self.assertTrue(np.all(a == ac))

    def test03(self):
        """Testing arange() with a `dtype`."""
        a = np.arange(self.N, dtype="i1")
        ac = bcolz.arange(self.N, dtype="i1", rootdir=self.rootdir)
        self.assertTrue(np.all(a == ac))


class arange_smallTest(arangeTest, TestCase):
    N = 10
    disk = False


class arange_bigTest(arangeTest, TestCase):
    N = 1e4
    disk = False


class arange_smallDiskTest(arangeTest, TestCase):
    N = 10
    disk = True


class arange_bigDiskTest(arangeTest, TestCase):
    N = 1e4
    disk = True


class constructorTest(MayBeDiskTest):

    def test00(self):
        """Testing carray constructor with an int32 `dtype`."""
        a = np.arange(self.N)
        ac = bcolz.carray(a, dtype='i4', rootdir=self.rootdir)
        self.assertTrue(ac.dtype == np.dtype('i4'))
        a = a.astype('i4')
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test01a(self):
        """Testing zeros() constructor."""
        a = np.zeros(self.N)
        ac = bcolz.zeros(self.N, rootdir=self.rootdir)
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test01b(self):
        """Testing zeros() constructor, with a `dtype`."""
        a = np.zeros(self.N, dtype='i4')
        ac = bcolz.zeros(self.N, dtype='i4', rootdir=self.rootdir)
        # print "dtypes-->", a.dtype, ac.dtype
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test01c(self):
        """Testing zeros() constructor, with a string type."""
        a = np.zeros(self.N, dtype='S5')
        ac = bcolz.zeros(self.N, dtype='S5', rootdir=self.rootdir)
        # print "ac-->", `ac`
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test02a(self):
        """Testing ones() constructor."""
        a = np.ones(self.N)
        ac = bcolz.ones(self.N, rootdir=self.rootdir)
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test02b(self):
        """Testing ones() constructor, with a `dtype`."""
        a = np.ones(self.N, dtype='i4')
        ac = bcolz.ones(self.N, dtype='i4', rootdir=self.rootdir)
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test02c(self):
        """Testing ones() constructor, with a string type"""
        a = np.ones(self.N, dtype='S3')
        ac = bcolz.ones(self.N, dtype='S3', rootdir=self.rootdir)
        # print "a-->", a, ac
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test03a(self):
        """Testing fill() constructor."""
        a = np.ones(self.N)
        ac = bcolz.fill(self.N, 1, rootdir=self.rootdir)
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test03b(self):
        """Testing fill() constructor, with a `dtype`."""
        a = np.ones(self.N, dtype='i4')*3
        ac = bcolz.fill(self.N, 3, dtype='i4', rootdir=self.rootdir)
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))

    def test03c(self):
        """Testing fill() constructor, with a string type"""
        a = np.ones(self.N, dtype='S3')
        ac = bcolz.fill(self.N, "1", dtype='S3', rootdir=self.rootdir)
        # print "a-->", a, ac
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac[:]))


class constructorSmallTest(constructorTest, TestCase):
    N = 10


class constructorSmallDiskTest(constructorTest, TestCase):
    N = 10
    disk = True


class constructorBigTest(constructorTest, TestCase):
    N = 50000


class constructorBigDiskTest(constructorTest, TestCase):
    N = 50000
    disk = True


class dtypesTest(TestCase):

    def test00(self):
        """Testing carray constructor with a float32 `dtype`."""
        a = np.arange(10)
        ac = bcolz.carray(a, dtype='f4')
        self.assertTrue(ac.dtype == np.dtype('f4'))
        a = a.astype('f4')
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac))

    def test01(self):
        """Testing carray constructor with a `dtype` with an empty input."""
        a = np.array([], dtype='i4')
        ac = bcolz.carray([], dtype='f4')
        self.assertTrue(ac.dtype == np.dtype('f4'))
        a = a.astype('f4')
        self.assertTrue(a.dtype == ac.dtype)
        self.assertTrue(np.all(a == ac))

    def test02(self):
        """Testing carray constructor with a plain compound `dtype`."""
        dtype = np.dtype("f4,f8")
        a = np.ones(30000, dtype=dtype)
        ac = bcolz.carray(a, dtype=dtype)
        self.assertTrue(ac.dtype == dtype)
        self.assertTrue(a.dtype == ac.dtype)
        # print "ac-->", `ac`
        assert_array_equal(a, ac[:], "Arrays are not equal")

    def test03(self):
        """Testing carray constructor with a nested compound `dtype`."""
        dtype = np.dtype([('f1', [('f1', 'i2'), ('f2', 'i4')])])
        a = np.ones(3000, dtype=dtype)
        ac = bcolz.carray(a, dtype=dtype)
        self.assertTrue(ac.dtype == dtype)
        self.assertTrue(a.dtype == ac.dtype)
        # print "ac-->", `ac`
        assert_array_equal(a, ac[:], "Arrays are not equal")

    def test04(self):
        """Testing carray constructor with a string `dtype`."""
        a = np.array(["ale", "e", "aco"], dtype="S4")
        ac = bcolz.carray(a, dtype='S4')
        self.assertTrue(ac.dtype == np.dtype('S4'))
        self.assertTrue(a.dtype == ac.dtype)
        # print "ac-->", `ac`
        assert_array_equal(a, ac, "Arrays are not equal")

    def test05(self):
        """Testing carray constructor with a unicode `dtype`."""
        a = np.array([u"aŀle", u"eñe", u"açò"], dtype="U4")
        ac = bcolz.carray(a, dtype='U4')
        self.assertTrue(ac.dtype == np.dtype('U4'))
        self.assertTrue(a.dtype == ac.dtype)
        # print "ac-->", `ac`
        assert_array_equal(a, ac, "Arrays are not equal")

    def test06(self):
        """Testing carray constructor with an object `dtype`."""
        dtype = np.dtype("object")
        a = np.array(["ale", "e", "aco"], dtype=dtype)
        ac = bcolz.carray(a, dtype=dtype)
        self.assertEqual(ac.dtype, dtype)
        self.assertEqual(a.dtype, ac.dtype)
        assert_array_equal(a, ac, "Arrays are not equal")

    def test07(self):
        """Checking carray constructor from another carray."""
        types = [np.int8, np.int16, np.int32, np.int64,
                 np.uint8, np.uint16, np.uint32, np.uint64,
                 np.float16, np.float32, np.float64,
                 np.complex64, np.complex128]
        if hasattr(np, 'float128'):
            types.extend([np.float128, np.complex256])
        shapes = [(10,), (10, 10), (10, 10, 10)]
        for shape in shapes:
            for t in types:
                a = bcolz.zeros(shape, t)
                b = bcolz.carray(a)
                self.assertEqual(a.dtype, b.dtype)
                self.assertEqual(a.shape, b.shape)
                self.assertEqual(a.shape, shape)


class largeCarrayTest(MayBeDiskTest):

    def test00(self):
        """Creating an extremely large carray (> 2**32) in memory."""

        cn = bcolz.zeros(5e9, dtype="i1")
        self.assertTrue(len(cn) == long(5e9))

        # Now check some accesses
        cn[1] = 1
        self.assertTrue(cn[1] == 1)
        cn[int(2e9)] = 2
        self.assertTrue(cn[int(2e9)] == 2)
        cn[long(3e9)] = 3
        self.assertTrue(cn[long(3e9)] == 3)
        cn[-1] = 4
        self.assertTrue(cn[-1] == 4)

        self.assertTrue(cn.sum() == 10)

    def test01(self):
        """Creating an extremely large carray (> 2**32) on disk."""

        cn = bcolz.zeros(5e9, dtype="i1", rootdir=self.rootdir)
        self.assertTrue(len(cn) == long(5e9))

        # Now check some accesses
        cn[1] = 1
        self.assertTrue(cn[1] == 1)
        cn[int(2e9)] = 2
        self.assertTrue(cn[int(2e9)] == 2)
        cn[long(3e9)] = 3
        self.assertTrue(cn[long(3e9)] == 3)
        cn[-1] = 4
        self.assertTrue(cn[-1] == 4)

        self.assertTrue(cn.sum() == 10)

    def test02(self):
        """Opening an extremely large carray (> 2**32) on disk."""

        if not self.disk:
            raise SkipTest
        # Create the array on-disk
        cn = bcolz.zeros(5e9, dtype="i1", rootdir=self.rootdir)
        self.assertTrue(len(cn) == int(5e9))
        # Reopen it from disk
        cn = bcolz.carray(rootdir=self.rootdir)
        self.assertTrue(len(cn) == int(5e9))

        # Now check some accesses
        cn[1] = 1
        self.assertTrue(cn[1] == 1)
        cn[int(2e9)] = 2
        self.assertTrue(cn[int(2e9)] == 2)
        cn[long(3e9)] = 3
        self.assertTrue(cn[long(3e9)] == 3)
        cn[-1] = 4
        self.assertTrue(cn[-1] == 4)

        self.assertTrue(cn.sum() == 10)


@skipUnless(is_64bit and common.heavy, "not 64bit or not --heavy")
class largeCarrayMemoryTest(largeCarrayTest, TestCase):
    disk = False


@skipUnless(is_64bit and common.heavy, "not 64bit or not --heavy")
class largeCarrayDiskTest(largeCarrayTest, TestCase):
    disk = True


class persistenceTest(MayBeDiskTest, TestCase):
    disk = True

    def test01a(self):
        """Creating a carray in "r" mode."""

        N = 10000
        self.assertRaises(IOError, bcolz.zeros,
                          N, dtype="i1", rootdir=self.rootdir, mode='r')

    def test01b(self):
        """Creating a carray in "w" mode."""

        N = 50000
        cn = bcolz.zeros(N, dtype="i1", rootdir=self.rootdir)
        self.assertTrue(len(cn) == N)

        cn = bcolz.zeros(N-2, dtype="i1", rootdir=self.rootdir, mode='w')
        self.assertTrue(len(cn) == N-2)

        # Now check some accesses (no errors should be raised)
        cn.append([1, 1])
        self.assertTrue(len(cn) == N)
        cn[1] = 2
        self.assertTrue(cn[1] == 2)

    def test01c(self):
        """Creating a carray in "a" mode."""

        N = 30003
        cn = bcolz.zeros(N, dtype="i1", rootdir=self.rootdir)
        self.assertTrue(len(cn) == N)

        self.assertRaises(IOError, bcolz.zeros,
                          N-2, dtype="i1", rootdir=self.rootdir, mode='a')

    def test02a(self):
        """Opening a carray in "r" mode."""

        N = 10001
        cn = bcolz.zeros(N, dtype="i1", rootdir=self.rootdir)
        self.assertTrue(len(cn) == N)

        cn = bcolz.carray(rootdir=self.rootdir, mode='r')
        self.assertTrue(len(cn) == N)

        # Now check some accesses
        self.assertRaises(IOError, cn.__setitem__, 1, 1)
        self.assertRaises(IOError, cn.append, 1)

    def test02b(self):
        """Opening a carray in "w" mode."""

        N = 100001
        cn = bcolz.zeros(N, dtype="i1", rootdir=self.rootdir)
        self.assertTrue(len(cn) == N)

        cn = bcolz.carray(rootdir=self.rootdir, mode='w')
        self.assertTrue(len(cn) == 0)

        # Now check some accesses (no errors should be raised)
        cn.append([1, 1])
        self.assertTrue(len(cn) == 2)
        cn[1] = 2
        self.assertTrue(cn[1] == 2)

    def test02c(self):
        """Opening a carray in "a" mode."""

        N = 1000-1
        cn = bcolz.zeros(N, dtype="i1", rootdir=self.rootdir)
        self.assertTrue(len(cn) == N)

        cn = bcolz.carray(rootdir=self.rootdir, mode='a')
        self.assertTrue(len(cn) == N)

        # Now check some accesses (no errors should be raised)
        cn.append([1, 1])
        self.assertTrue(len(cn) == N+2)
        cn[1] = 2
        self.assertTrue(cn[1] == 2)
        cn[N+1] = 3
        self.assertTrue(cn[N+1] == 3)


class bloscCompressorsTest(MayBeDiskTest, TestCase):

    def tearDown(self):
        # Restore defaults
        bcolz.cparams.setdefaults(clevel=5, shuffle=True, cname='blosclz')
        MayBeDiskTest.tearDown(self)

    def test00(self):
        """Testing all available compressors in small arrays"""
        a = np.arange(20)
        cnames = bcolz.blosc_compressor_list()
        if common.verbose:
            print("Checking compressors:", cnames)
        # print "\nsize b uncompressed-->", a.size * a.dtype.itemsize
        for cname in cnames:
            b = bcolz.carray(a, rootdir=self.rootdir,
                             cparams=bcolz.cparams(clevel=9, cname=cname))
            # print "size b compressed  -->", b.cbytes, "with '%s'"%cname
            self.assertTrue(sys.getsizeof(b) > b.nbytes,
                            "compression does not seem to have any overhead")
            assert_array_equal(a, b[:], "Arrays are not equal")
            # Remove the array on disk before trying with the next one
            if self.disk:
                common.remove_tree(self.rootdir)

    def test01a(self):
        """Testing all available compressors in big arrays (setdefaults)"""
        a = np.arange(1e5)
        cnames = bcolz.blosc_compressor_list()
        if common.verbose:
            print("Checking compressors:", cnames)
        # print "\nsize b uncompressed-->", a.size * a.dtype.itemsize
        for cname in cnames:
            bcolz.cparams.setdefaults(clevel=9, cname=cname)
            b = bcolz.carray(a, rootdir=self.rootdir)
            # print "size b compressed  -->", b.cbytes, "with '%s'"%cname
            self.assertTrue(sys.getsizeof(b) < b.nbytes,
                            "carray does not seem to compress at all")
            assert_array_equal(a, b[:], "Arrays are not equal")
            # Remove the array on disk before trying with the next one
            if self.disk:
                common.remove_tree(self.rootdir)

    def test01b(self):
        """Testing all available compressors in big arrays (bcolz.defaults)"""
        a = np.arange(1e5)
        cnames = bcolz.blosc_compressor_list()
        if common.verbose:
            print("Checking compressors:", cnames)
        # print "\nsize b uncompressed-->", a.size * a.dtype.itemsize
        for cname in cnames:
            bcolz.defaults.cparams = {
                'clevel': 9, 'shuffle': True, 'cname': cname}
            b = bcolz.carray(a, rootdir=self.rootdir)
            # print "size b compressed  -->", b.cbytes, "with '%s'"%cname
            self.assertTrue(sys.getsizeof(b) < b.nbytes,
                            "carray does not seem to compress at all")
            assert_array_equal(a, b[:], "Arrays are not equal")
            # Remove the array on disk before trying with the next one
            if self.disk:
                common.remove_tree(self.rootdir)
        # Restore defaults
        bcolz.defaults.cparams = {
            'clevel': 5, 'shuffle': True, 'cname': 'blosclz'}


class compressorsMemoryTest(bloscCompressorsTest, TestCase):
    disk = False


class compressorsDiskTest(bloscCompressorsTest, TestCase):
    disk = True


class reprTest(TestCase):
    def test_datetime_carray_day(self):
        ct = carray(np.array(['2010-01-01', '2010-01-02'],
                             dtype='datetime64[D]'))
        result = repr(ct)
        self.assertTrue("['2010-01-01' '2010-01-02']" in result)

    def test_datetime_carray_nanos(self):
        x = ['2014-12-29T17:57:59.000000123',
             '2014-12-29T17:57:59.000000456']
        ct = carray(np.array(x, dtype='datetime64[ns]'))
        result = repr(ct)
        for el in x:
            self.assertTrue(el in result)

class PurgeDiskArrayTest(MayBeDiskTest, TestCase):
    disk = True

    def test_purge(self):
        b = bcolz.arange(1e2, rootdir=self.rootdir)
        b.purge()
        self.assertFalse(os.path.isdir(self.rootdir))

    def test_purge_fails_for_missing_directory(self):
        b = bcolz.arange(1e2, rootdir=self.rootdir)
        shutil.rmtree(self.rootdir)
        # OSError is fairly un-specific, but better than nothing
        self.assertRaises(OSError, b.purge)

class PurgeMemoryArrayTest(MayBeDiskTest, TestCase):
    disk = False

    def test_purge(self):
        b = bcolz.arange(1e2, rootdir=self.rootdir)
        # this should work and should be a noop
        b.purge()

class reprDiskTest(MayBeDiskTest,TestCase):
    disk = True

    def _create_expected(self, mode):
        expected = textwrap.dedent("""
                   carray((0,), float64)
                     nbytes: 0; cbytes: 16.00 KB; ratio: 0.00
                     cparams := cparams(clevel=5, shuffle=True, cname='blosclz')
                     rootdir := '%s'
                     mode    := '%s'
                   []
                   """ % (self.rootdir, mode)).strip()
        return expected


    def test_repr_disk_array_write(self):
        x = carray([], rootdir=self.rootdir, mode='w')
        expected = self._create_expected('w')
        self.assertEqual(expected, repr(x))

    def test_repr_disk_array_read(self):
        x = carray([], rootdir=self.rootdir, mode='r')
        expected = self._create_expected('r')
        self.assertEqual(expected, repr(x))

    def test_repr_disk_array_append(self):
        x = carray([], rootdir=self.rootdir, mode='w')
        y = carray(rootdir=self.rootdir)
        expected = self._create_expected('a')
        self.assertEqual(expected, repr(y))

class chunksIterTest(MayBeDiskTest):
    def test00(self):
        """Testing chunk iterator"""
        _dtype = 'int64'
        chunklen_ = 10
        N = 1e2 + 3
        a = np.arange(N, dtype=_dtype)
        b = bcolz.carray(a, dtype=_dtype, chunklen=chunklen_,
                         rootdir=self.rootdir)
        # print 'nchunks', b.nchunks
        for n, chunk_ in enumerate(b.chunks):
            # print 'chunk nr.', n, '-->', chunk_
            assert_array_equal(chunk_[0:chunklen_],
                               a[n * chunklen_:(n + 1) * chunklen_],
                               "iter chunks not working correctly")

        self.assertEquals(n, len(a) // chunklen_ - 1)


class chunksIterMemoryTest(chunksIterTest, TestCase):
    disk = False


class chunksIterDiskTest(chunksIterTest, TestCase):
    disk = True

class ContextManagerTest(MayBeDiskTest, TestCase):
    disk = True

    def test_with_statement_flushes(self):

        with carray([], rootdir=self.rootdir, mode='w') as x:
            x.append(1)
        received = np.array(carray(rootdir=self.rootdir))
        expected = np.array([1])
        assert_array_equal(expected, received)

class nleftoversTest(TestCase):

    def test_empty(self):
        a = carray([])
        self.assertEquals(0, a.nleftover)

    def test_one(self):
        a = carray([1])
        self.assertEqual(1, a.nleftover)

    def test_beyond_one(self):
        a = carray(np.zeros(2049), chunklen=2048)
        self.assertEqual(1, a.nleftover)


class LeftoverTest(MayBeDiskTest):

    def test_leftover_ptr(self):
        typesize = 8
        items = 7
        a = carray([i for i in range(items)], dtype='i8', rootdir=self.rootdir)
        for i in range(items):
            out = ctypes.c_int64.from_address(a.leftover_ptr + (i * typesize))
            self.assertEqual(i, out.value)

    def test_leftover_ptr_after_chunks(self):
        typesize = 4
        items = 108
        chunklen = 100
        a = carray([i for i in range(items)], chunklen=chunklen, dtype='i4',
                   rootdir=self.rootdir)
        for i in range(items % chunklen):
            out = ctypes.c_int32.from_address(a.leftover_ptr + (i * typesize))
            self.assertEqual(chunklen + i, out.value)

    def test_leftover_array(self):
        items = 7
        a = carray([i for i in range(items)], dtype='i4', rootdir=self.rootdir)
        reference = np.array([0, 1, 2, 3, 4, 5, 6], dtype='int32')
        out = a.leftover_array[:items]
        assert_array_equal(reference, out)

    def test_leftover_bytes(self):
        typesize = 8
        items = 9
        a = carray([i for i in range(items)], dtype='i8', rootdir=self.rootdir)
        self.assertEqual(a.leftover_bytes, items * typesize)

    def test_leftover_elements(self):
        items = 9
        a = carray([i for i in range(items)], dtype='i8', rootdir=self.rootdir)
        self.assertEqual(a.leftover_elements, items)


class LeftoverMemoryTest(LeftoverTest, TestCase):
    disk = False


class LeftoverDiskTest(LeftoverTest, TestCase):
    disk = True

    def test_leftover_ptr_create_flush_open(self):
        typesize = 8
        items = 120
        chunklen = 50
        n_leftovers = items % chunklen
        n_chunks = items // chunklen

        a = carray([i for i in range(items)], chunklen=chunklen, dtype='i8',
                   rootdir=self.rootdir)
        a.flush()

        b = carray(rootdir=self.rootdir)
        for i in range(n_leftovers):
            out = ctypes.c_int32.from_address(b.leftover_ptr + (i * typesize))
            self.assertEqual((n_chunks * chunklen) + i, out.value)

    def test_leftover_ptr_with_statement_create_open(self):
        typesize = 8
        items = 120
        chunklen = 50
        n_leftovers = items % chunklen
        n_chunks = items // chunklen

        ca = carray([], chunklen=chunklen, dtype='i8', rootdir=self.rootdir)
        with ca as a:
            for i in range(items):
                a.append(i)

        b = carray(rootdir=self.rootdir)
        for i in range(n_leftovers):
            out = ctypes.c_int32.from_address(b.leftover_ptr + (i * typesize))
            self.assertEqual((n_chunks * chunklen) + i, out.value)


if __name__ == '__main__':
    unittest.main(verbosity=2)


# Local Variables:
# mode: python
# tab-width: 4
# fill-column: 72
# End:



# Benchmark for evaluate best ways to read from a PyTables Table

import bcolz
import tables as tb
import numpy as np
from time import time

filepath = 'fromhdf5.h5'
nodepath = '/ctable'
NR = int(1e6)
NC = 10
dsize = (NR * NC * 4) / 2. ** 30

bcolz.cparams.setdefaults(clevel=5)

a = bcolz.arange(NR, dtype='i4')
#ra = np.rec.fromarrays([a]*NC, names=['f%d'%i for i in range(NC)])
ra = bcolz.ctable((a,)*NC)[:]

t0 = time()
f = tb.open_file(filepath, "w")
f.create_table(f.root, nodepath[1:], ra)
f.close()
tt = time() - t0
print("time for storing the HDF5 table: %.2f (%.2f GB/s)" % (tt, dsize / tt))

# Using an iterator
t0 = time()
f = tb.open_file(filepath)
t = f.get_node(nodepath)
t = bcolz.fromiter((r[:] for r in t), dtype=t.dtype, count=len(t))
f.close()
tt = time() - t0
print("time with fromiter: %.2f (%.2f GB/s)" % (tt, dsize / tt))

# Using blocked read
t0 = time()
f = tb.open_file(filepath)
t = f.get_node(nodepath)
names = t.colnames
dtypes = [dt[0] for dt in t.dtype.fields.values()]
cols = [np.zeros(0, dtype=dt) for dt in dtypes]
ct = bcolz.ctable(cols, names)
bs = t._v_chunkshape[0]
for i in xrange(0, len(t), bs):
    ct.append(t[i:i+bs])
f.close()
tt = time() - t0
print("time with blocked read: %.2f (%.2f GB/s)" % (tt, dsize / tt))

# Using generic implementation
t0 = time()
t = bcolz.ctable.fromhdf5(filepath, nodepath)
tt = time() - t0
print("time with fromhdf5: %.2f (%.2f GB/s)" % (tt, dsize / tt))


#print(repr(ct))




########################################################################
#
#       License: BSD
#       Created: July 15, 2014
#       Author:  Francesc Alted - francesc@blosc.io
#
########################################################################

"""Defaults for different bcolz parameters.
"""

from __future__ import absolute_import

import bcolz


class Defaults(object):
    """Class to taylor the setters and getters of default values."""

    def __init__(self):
        self.choices = {}

        # Choices setup
        self.choices['eval_out_flavor'] = ("carray", "numpy")
        self.choices['eval_vm'] = ("numexpr", "python")

    def check_choices(self, name, value):
        if value not in self.choices[name]:
            raise ValueError(
                "'%s' is incorrect value for '%s' default" % (value, name))

    def check_cparams(self, value):
        if not isinstance(value, dict):
            raise ValueError(
                "this needs to be a dictionary and you "
                "passed '%s' " % type(value))
        if ('clevel' not in value or
                'shuffle' not in value or
                'cname' not in value):
            raise ValueError(
                "The dictionary must have the next entries: "
                "'clevel', 'shuffle' and 'cname'")
        clevel, shuffle, cname = bcolz.cparams._checkparams(
            value['clevel'], value['shuffle'], value['cname'])
        return {'clevel': clevel, 'shuffle': shuffle, 'cname': cname}

    #
    # Properties start here...
    #

    @property
    def eval_vm(self):
        return self.__eval_vm

    @eval_vm.setter
    def eval_vm(self, value):
        self.check_choices('eval_vm', value)
        if value == "numexpr" and not bcolz.numexpr_here:
            raise (ValueError,
                   "cannot use `numexpr` virtual machine "
                   "(minimum required version is probably not installed)")
        self.__eval_vm = value

    @property
    def eval_out_flavor(self):
        return self.__eval_out_flavor

    @eval_out_flavor.setter
    def eval_out_flavor(self, value):
        self.check_choices('eval_out_flavor', value)
        self.__eval_out_flavor = value

    @property
    def cparams(self):
        return self.__cparams

    @cparams.setter
    def cparams(self, value):
        self.__cparams = self.check_cparams(value)


defaults = Defaults()


# Default values start here...

defaults.eval_out_flavor = "carray"
"""
The flavor for the output object in `eval()`.  It can be 'carray' or
'numpy'.  Default is 'carray'.
"""

defaults.eval_vm = "numexpr" if bcolz.numexpr_here else "python"
"""
The virtual machine to be used in computations (via `eval`).  It can
be 'numexpr' or 'python'.  Default is 'numexpr', if it is installed.
If not, then the default is 'python'.
"""

defaults.cparams = {'clevel': 5, 'shuffle': True, 'cname': 'blosclz'}
"""
The defaults for parameters used in compression.  You can change
them more comfortably by using the `cparams.setdefaults()` method.
"""



from time import time

import numpy as np

import bcolz


N = 1e8
# a = np.arange(N, dtype='f8')
a = np.random.randint(0, 10, N).astype('bool')

t0 = time()
sa = a.sum()
print("Time sum() numpy --> %.3f" % (time() - t0))

t0 = time()
ac = bcolz.carray(a, cparams=bcolz.cparams(9))
print("Time carray conv --> %.3f" % (time() - t0))
print("ac-->", repr(ac))

t0 = time()
sac = ac.sum()
#sac = ac.sum(dtype=np.dtype('i8'))
print("Time sum() carray --> %.3f" % (time() - t0))

# t0 = time()
# sac = sum(i for i in ac)
# print "Time sum() carray (iter) --> %.3f" % (time()-t0)

print("sa, sac-->", sa, sac, type(sa), type(sac))
assert (sa == sac)



# Benchmark for evaluate best ways to write to a PyTables Table

import os
import bcolz
import tables as tb
import numpy as np
from time import time

filepath = 'tohdf5.h5'
nodepath = '/ctable'
NR = int(1e6)
NC = 10
dsize = (NR * NC * 4) / 2. ** 30

bcolz.cparams.setdefaults(clevel=5)

a = bcolz.arange(NR, dtype='i4')
ct = bcolz.ctable((a,)*NC)

# Row-by-row using an iterator
# t0 = time()
# f = tb.open_file(filepath, 'w')
# t = f.create_table(f.root, nodepath[1:], ct.dtype)
# for row in ct:
#     t.append([row])
# f.close()
# tt = time() - t0
# print("time with iterator: %.2f (%.2f GB/s)" % (tt, dsize / tt))

# Using blocked write
t0 = time()
f = tb.open_file(filepath, 'w')
t = f.create_table(f.root, nodepath[1:], ct.dtype)
for block in bcolz.iterblocks(ct):
    t.append(block)
f.close()
tt = time() - t0
print("time with blocked write: %.2f (%.2f GB/s)" % (tt, dsize / tt))

# Using generic implementation
os.remove(filepath)
t0 = time()
#ct.tohdf5(filepath, nodepath)
ct.tohdf5(filepath, nodepath, cname="blosc:blosclz")
tt = time() - t0
print("time with tohdf5: %.2f (%.2f GB/s)" % (tt, dsize / tt))


#print(repr(ct))




########################################################################
#
# License: BSD
#       Created: August 16, 2012
#       Author:  Francesc Alted - francesc@blosc.io
#
########################################################################

from __future__ import absolute_import

# The minimum version of Cython required for generating extensions
min_cython_version = '0.22'
# The minimum version of NumPy required
min_numpy_version = '1.7'
# The minimum version of Numexpr (optional)
min_numexpr_version = '1.4.1'

import sys
import os
import glob
try:
    import setuptools
    from setuptools import setup, Extension
    setuptools.dist.Distribution(dict(setup_requires=['cython>=' + min_cython_version, 'numpy>='+min_numpy_version]))
except:
    from distutils.core import setup, Extension

import textwrap
import re, platform


########### Some utils for version checking ################

# Some functions for showing errors and warnings.
def _print_admonition(kind, head, body):
    tw = textwrap.TextWrapper(
        initial_indent='   ', subsequent_indent='   ')

    print( ".. %s:: %s" % (kind.upper(), head))
    for line in tw.wrap(body):
        print(line)


def exit_with_error(head, body=''):
    _print_admonition('error', head, body)
    sys.exit(1)


def print_warning(head, body=''):
    _print_admonition('warning', head, body)


def check_import(pkgname, pkgver):
    try:
        mod = __import__(pkgname)
    except ImportError:
        exit_with_error(
            "You need %(pkgname)s %(pkgver)s or greater to run bcolz!"
            % {'pkgname': pkgname, 'pkgver': pkgver})
    else:
        if mod.__version__ < pkgver:
            exit_with_error(
                "You need %(pkgname)s %(pkgver)s or greater to run bcolz!"
                % {'pkgname': pkgname, 'pkgver': pkgver})

    print("* Found %(pkgname)s %(pkgver)s package installed."
          % {'pkgname': pkgname, 'pkgver': mod.__version__})
    globals()[pkgname] = mod


########### Check versions ##########

# Check for Python
if sys.version_info[0] == 2:
    if sys.version_info[1] < 6:
        exit_with_error("You need Python 2.6 or greater to run bcolz!")
    if sys.version_info[1] < 7:
        try:
            import unittest2
        except ImportError:
            print("You need unittest2 for running bcolz tests with Python 2.6!")
elif sys.version_info[0] == 3:
    if sys.version_info[1] < 2:
        exit_with_error("You need Python 3.2 or greater to run bcolz!")
else:
    exit_with_error("You need Python 2.6/3.2 or greater to run bcolz!")

# Check if Cython is installed or not (requisite)
try:
    import Cython
    cur_cython_version = Cython.__version__
    from Cython.Distutils import build_ext
except:
    exit_with_error(
        "You need %(pkgname)s %(pkgver)s or greater to compile bcolz!"
        % {'pkgname': 'Cython', 'pkgver': min_cython_version})

if cur_cython_version < min_cython_version:
    exit_with_error(
        "At least Cython %s is needed so as to generate extensions!"
        % (min_cython_version))
else:
    print("* Found %(pkgname)s %(pkgver)s package installed."
          % {'pkgname': 'Cython', 'pkgver': cur_cython_version})

# Prevent numpy from thinking it is still in its setup process:
__builtins__.__NUMPY_SETUP__ = False

# Check for NumPy
check_import('numpy', min_numpy_version)

# Check for Numexpr
numexpr_here = False
try:
    import numexpr
except ImportError:
    print_warning(
        "Numexpr is not installed.  For faster bcolz operation, "
        "please consider installing it.")
else:
    if numexpr.__version__ >= min_numexpr_version:
        numexpr_here = True
        print("* Found %(pkgname)s %(pkgver)s package installed."
              % {'pkgname': 'numexpr', 'pkgver': numexpr.__version__})
    else:
        print_warning(
            "Numexpr %s installed, but version is not >= %s.  "
            "Disabling support for it." % (
                numexpr.__version__, min_numexpr_version))

########### End of checks ##########


# bcolz version
VERSION = open('VERSION').read().strip()
# Create the version.py file
open('bcolz/version.py', 'w').write('__version__ = "%s"\n' % VERSION)


# Global variables
CFLAGS = os.environ.get('CFLAGS', '').split()
LFLAGS = os.environ.get('LFLAGS', '').split()
# Allow setting the Blosc dir if installed in the system
BLOSC_DIR = os.environ.get('BLOSC_DIR', '')

# Sources & libraries
inc_dirs = ['bcolz']
lib_dirs = []
libs = []
def_macros = []
sources = ["bcolz/carray_ext.pyx"]

# Include NumPy header dirs
from numpy.distutils.misc_util import get_numpy_include_dirs
inc_dirs.extend(get_numpy_include_dirs())
optional_libs = []

# Handle --blosc=[PATH] --lflags=[FLAGS] --cflags=[FLAGS]
args = sys.argv[:]
for arg in args:
    if arg.find('--blosc=') == 0:
        BLOSC_DIR = os.path.expanduser(arg.split('=')[1])
        sys.argv.remove(arg)
    if arg.find('--lflags=') == 0:
        LFLAGS = arg.split('=')[1].split()
        sys.argv.remove(arg)
    if arg.find('--cflags=') == 0:
        CFLAGS = arg.split('=')[1].split()
        sys.argv.remove(arg)

if not BLOSC_DIR:
    # Compiling everything from sources
    # Blosc + BloscLZ sources
    sources += glob.glob('c-blosc/blosc/*.c')
    # LZ4 sources
    sources += glob.glob('c-blosc/internal-complibs/lz4*/*.c')
    # Snappy sources
    sources += glob.glob('c-blosc/internal-complibs/snappy*/*.cc')
    # Zlib sources
    sources += glob.glob('c-blosc/internal-complibs/zlib*/*.c')
    # Finally, add all the include dirs...
    inc_dirs += [os.path.join('c-blosc', 'blosc')]
    inc_dirs += glob.glob('c-blosc/internal-complibs/*')
    # ...and the macros for all the compressors supported
    def_macros += [('HAVE_LZ4', 1), ('HAVE_SNAPPY', 1), ('HAVE_ZLIB', 1)]
else:
    inc_dirs += [os.path.join(BLOSC_DIR, 'include')]
    lib_dirs += [os.path.join(BLOSC_DIR, 'lib')]
    libs += ['blosc']

# Add -msse2 flag for optimizing shuffle in included c-blosc
# (only necessary for 32-bit Intel architectures)
if os.name == 'posix' and re.match("i.86", platform.machine()) != None:
    CFLAGS.append("-msse2")


classifiers = """\
Development Status :: 4 - Beta
Intended Audience :: Developers
Intended Audience :: Information Technology
Intended Audience :: Science/Research
License :: OSI Approved :: BSD License
Programming Language :: Python
Topic :: Software Development :: Libraries :: Python Modules
Operating System :: Microsoft :: Windows
Operating System :: Unix
"""
setup(name="bcolz",
      version=VERSION,
      description='columnar and compressed data containers.',
      long_description="""\

bcolz provides columnar and compressed data containers.  Column
storage allows for efficiently querying tables with a large number of
columns.  It also allows for cheap addition and removal of column.  In
addition, bcolz objects are compressed by default for reducing
memory/disk I/O needs.  The compression process is carried out
internally by Blosc, a high-performance compressor that is optimized
for binary data.

""",
      classifiers=filter(None, classifiers.split("\n")),
      author='Francesc Alted',
      author_email='francesc@blosc.io',
      maintainer='Francesc Alted',
      maintainer_email='francesc@blosc.io',
      url='https://github.com/Blosc/bcolz',
      license='http://www.opensource.org/licenses/bsd-license.php',
      # It is better to upload manually to PyPI
      #download_url = 'http://github.com/downloads/Blosc/bcolz/python-bcolz
      # -%s.tar.gz' % (VERSION,),
      platforms=['any'],
      install_requires=['cython>=' + min_cython_version, 'numpy>='+min_numpy_version],
      cmdclass={'build_ext': build_ext},
      ext_modules=[
          Extension("bcolz.carray_ext",
                    include_dirs=inc_dirs,
                    define_macros=def_macros,
                    sources=sources,
                    library_dirs=lib_dirs,
                    libraries=libs,
                    extra_link_args=LFLAGS,
                    extra_compile_args=CFLAGS)
      ],
      packages=['bcolz', 'bcolz.tests'],
      package_data={'bcolz': ['carray_ext.pxd']},
)



from time import time

import numpy as np

import bcolz


N = 1e8
dtype = 'i4'

start, stop, step = 5, N, 4

t0 = time()
a = np.arange(start, stop, step, dtype=dtype)
print("Time numpy.arange() --> %.3f" % (time() - t0))

t0 = time()
ac = bcolz.arange(start, stop, step, dtype=dtype)
print("Time bcolsz.arange() --> %.3f" % (time() - t0))

print("ac-->", repr(ac))

# assert(np.all(a == ac))



# Benchmark for evaluate best ways to convert from a pandas dataframe
# (version with a mix of columns of ints and strings)

import bcolz
import pandas as pd
import numpy as np
from time import time

NR = int(1e6)
NC = 100

#bcolz.cparams.setdefaults(clevel=0)

print("Creating inputs...")
a = bcolz.arange(NR, dtype='i4')
s = bcolz.fromiter(("%d"%i for i in xrange(NR)), dtype='S7', count=NR)
df = pd.DataFrame.from_items((
    ('f%d'%i, a[:] if i < (NC//2) else s[:]) for i in range(NC)))

dsize = (NR * (NC//2) * (a.dtype.itemsize + s.dtype.itemsize)) / 2. ** 20

print("Performing benchmarks...")
# # Using an iterator (will get objects)
# t0 = time()
# names = list(df.columns.values)
# t = bcolz.ctable([df[key] for key in names], names)
# tt = time() - t0
# print("time with constructor: %.2f (%.2f MB/s)" % (tt, dsize / tt))
# print(repr(t.dtype))

# Using generic implementation
t0 = time()
t = bcolz.ctable.fromdataframe(df)
tt = time() - t0
print("time with fromdataframe: %.2f (%.2f MB/s)" % (tt, dsize / tt))
print(t.dtype)

